
// *********************************************************************
//
// $Source: /afs/desy.de/group/hera-b/repository/ARTE/arte/include/ArteTable.hh,v $
// $Revision: 1.14 $
// $Author: natasha $
// $Date: 1999/07/02 10:25:07 $
//
// Created: 09-03-97,   C. Borgmeier
//
// *********************************************************************

//
// This file is part of the C++ interface for Arte
// the HERA-B reconstruction and analysis tool
//
// This is the definition of the ArtePointer class.
//
// ArtePointer allows access to the Arte data tables in
// a pointer-like way.
//
//
// .SS Usage // A Pointer to an GESL is declared as
//
// .ft B
//      ArtePointer<GESL> geslPoint;
//
//
// .SS See Also
//
// ArteStructures.hh
//
// Author: Christoph Borgmeier
//

#ifndef _ARTE_TABLE_HH_
#define _ARTE_TABLE_HH_
#define _ARTE_HH_


#ifdef USE_GATE
#include <Gate/RtypesT.h>
#endif

#if !defined(__CINT__) 

#include <string>
#include <iterator>

#else   /* __CINT__ */

//typedef TString string;
#include <stddef.h>
#include <iterator.h>

#endif  /* __CINT__ */

#include <arte/PointerBase.hh>


#ifndef NO_ARTE
extern "C" void mmbrel_( const int& link,
			 const ArtePointerBase& e1,
			 const ArtePointerBase& e2 );
extern "C" void mmdrop_( int &link );
extern "C" void mmbook_( int &link, const int& );
#endif

template <class T> class ArteTable;
template <class T> class ArteTableIterator;

template <class T>
class ArtePointer: public ArtePointerBase
{

  friend inline bool operator< <>( ArtePointer<T> p1, ArtePointer<T> p2 );
  friend inline int  operator- <>( ArtePointer<T> p1, ArtePointer<T> p2 );
#ifdef USE_GATE
  friend void operator<< <>( TBuffer& buf, const ArtePointer<T> ap );
#endif
  friend class ArteTable<T>;
  friend class ArteTableIterator<T>;

private:
  class Pointer {};
public:
  typedef ArtePointer<T> self;
  typedef T value_type;
  typedef value_type* pointer;
  typedef value_type& reference;
  typedef const value_type& const_reference;
  
  ArtePointer( const ArtePointer<T>& index ): ArtePointerBase( index ) {}  
  ArtePointer(): ArtePointerBase( 0 ) {}
  
  inline reference operator*() const;
  inline T* operator->() const;

  inline void setInt( unsigned int index )
    { ArtePointerBase::operator=( ArtePointerBase( index ) ); }

#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1) || defined(CINTOBJECT)

  ArtePointer( unsigned int index ): ArtePointerBase( index ) {}

  inline ArtePointer<T>& operator++() { increment(); return *this; }
  inline ArtePointer<T> operator++(int) {
    ArtePointer<T> tmp = *this;
    increment(); return tmp; }

#if (!defined(NO_ARTE))

  inline static unsigned int nrow();
  // Returns the number of entries in the corresponding Arte table.
  
  inline static self append(const_reference e);

  inline static void* getLink();
  inline static int& getIntLink();
  inline static ArteLink& getLinkLink();

  inline static int ncol();
  // Return the width of the Arte table.

#endif /* NO_ARTE */
public:
#else
  ArtePointer( Pointer* ): ArtePointerBase( 0 ) {}

  bool operator!=( Pointer* ) const
    { return ArtePointerBase::operator!=( 0 ); }

protected:
#endif /* COMPAT */
  unsigned int index() const { return ArtePointerBase::index(); }

public:
  bool operator!=( const ArtePointer<T>& rhs ) const
    { return ArtePointerBase::operator!=( rhs ); }

  bool operator==( const ArtePointer<T>& rhs ) const
    { return !operator!=( rhs ); }

private:
  inline void* dereference( void* link ) const;
};


template<class T>
inline bool operator<( ArtePointer<T> p1, ArtePointer<T> p2 )
{
  return p1.index()<p2.index();
}


template<class T>
inline int operator-( ArtePointer<T> p1, ArtePointer<T> p2 )
{
  return p1.index()-p2.index();
}


#ifdef USE_GATE

template<class T>
inline void operator<<( TBuffer& buf, const ArtePointer<T> ap )
{
  buf << ap.index();
}


template<class T>
inline void operator>>( TBuffer& buf, ArtePointer<T>& ap )
{
  int idx;
  buf >> idx;
  ap.setInt( idx );
}


template<class T>
inline void operator>>( TBuffer& buf, ArteTableIterator<T>& ap )
{
  ArtePointer<T> p;
  int idx;
  buf >> idx;
  p.setInt( idx );
  ap = p;
}


#endif


//#ifndef __CINT__
template <class T>
class ArteTableIterator
{
  friend class ArteTable<T>;
  friend int operator==<>( const ArteTableIterator<T>&,
			   const ArteTableIterator<T>& );
  friend int operator-<>( const ArteTableIterator<T>&,
			  const ArteTableIterator<T>& );
  friend ArteTableIterator<T> operator-<>( const ArteTableIterator<T>& lhs,
					   int rhs );
public:
  typedef ArteTableIterator<T> self;
  typedef ArtePointer<T> value_type;
  typedef value_type* pointer;
#ifndef __CINT__
  typedef value_type& reference;
  typedef const value_type& const_reference;
  typedef random_access_iterator_tag iterator_category;
  typedef ptrdiff_t difference_type;
#else
  typedef ArtePointer<T>& reference;
  typedef const ArtePointer<T>& const_reference;
  typedef random_access_iterator_tag iterator_category;
  typedef int difference_type;
#endif

  inline ArteTableIterator( const ArtePointer<T>& rhs );
  ArteTableIterator()
    {
#ifdef NO_ARTE
      ptr.setInt( 0 );
#else
      ptr.setInt( 1 );
#endif
    }
  
  reference operator*()
    { return ptr; }

  const_reference operator*() const
    { return ptr; }

  self& operator++() {
    ptr.increment();
    return *this;
  }
  self operator++(int) {
    self tmp = *this;
    ptr.increment();
    return tmp;
  }
  self& operator--() {
    ptr.decrement();
    return *this;
  }
  self operator--(int) {
    self tmp = *this;
    ptr.decrement();
    return tmp;
  }
  self& operator+=( int offset ) { ptr+=offset; return *this; }

#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
  operator ArtePointer<T>() const { return ptr; }
  inline int valid() const;
  ArtePointer<T> next();
  inline static unsigned int nrow();
  inline T* operator->() const;
#else
protected:
#endif
  unsigned int index() const { return ptr.index(); }
  ArteTableIterator( unsigned int i )
    : ptr(*( ArteTable<T>::begin() + (i - 1) )) {}
private:
  ArtePointer<T> ptr;
};
//#endif

template<class T>
inline int operator==(const ArteTableIterator<T>& el1,
		      const ArteTableIterator<T>& el2)
{ return el1.ptr==el2.ptr; }


template <class T>
inline random_access_iterator_tag
iterator_category(const ArteTableIterator<T>&) {
  return random_access_iterator_tag();
}


template <class T>
inline ptrdiff_t*
distance_type(const ArteTableIterator<T>&) {
  return 0;
}


template <class T>
inline ArteTableIterator<T> operator+( const ArteTableIterator<T>& lhs,
				       int rhs )
{
  ArteTableIterator<T> tmp = lhs;
  tmp+=rhs;
  return tmp;
}


template <class T>
inline ArteTableIterator<T> operator-( const ArteTableIterator<T>& lhs,
				       int rhs )
{
  return operator+(lhs,-rhs);
}


template <class T>
inline int operator-( const ArteTableIterator<T>& lhs,
		      const ArteTableIterator<T>& rhs )
{
  return operator-( lhs.ptr, rhs.ptr );
}


//
// template class ArteTable
//
// STL compatible container for Arte tables
//
template <class T>
class ArteTable: public TObject
{
  friend class ArtePointer<T>;
public:
  typedef ArtePointer<T> value_type;
#ifndef __CINT__
  typedef value_type& reference;
  typedef const value_type& const_reference;

  typedef size_t size_type;
  typedef ptrdiff_t difference_type;
#else
  typedef ArtePointer<T>& reference;
  typedef const ArtePointer<T>& const_reference;

  typedef unsigned int size_type;
  typedef int difference_type;
#endif

  typedef ArteTableIterator<T> iterator;
  
//   this thing does not go through egcs 1.0
//   typedef reverse_bidirectional_iterator<const_iterator, value_type,
//                                          const_reference, difference_type>
//       const_reverse_iterator;

#ifndef __CINT__
  // this can become too long for CINT
  typedef reverse_bidirectional_iterator<iterator, value_type, reference,
    difference_type>
  reverse_iterator;
#endif /* __CINT__ */
public:
  static iterator begin();
  
  static iterator end();

  static unsigned int size();
  static ArtePointer<T> push_back(const T& e);
#ifndef NO_ARTE
  static void book( int num = 1 )
    { mmbook_( *static_cast<int*>(getLink()), num ); }
  static bool booked() { return *static_cast<int*>(getLink())!=0; }
  static void free() { mmdrop_( *static_cast<int*>(getLink()) ); }
  static void clear() { mmdrop_( *static_cast<int*>(getLink()) ); }
#endif
#ifndef __CINT__
  static string getName();
#endif

  ArteTable() {};
  
public:
#ifdef USE_ROOT
  virtual void Print( Option_t* = "" );
#endif

  static bool isArteTable();
  
#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
  inline static unsigned int nrow() { return size(); }
private:
#endif

  static void* getLink();

#ifndef NO_ARTE
private:
  static int ncol() { return gcbank_.iq[*static_cast<int*>(getLink())]; }
  // Return the width of the Arte table
#endif
#if defined(USE_GATE) && defined(USE_ROOT)
  ClassDefT(ArteTable<T>,1)
#endif
};


#if defined(USE_GATE) && defined(USE_ROOT)
ClassDefT2(ArteTable,T)
#endif


template<class T>
inline ArteTableIterator<T>::ArteTableIterator( const ArtePointer<T>& p )
  : ptr( p )
{
}
  
  
//
// Relationpointer Base
//
template<class Relation, class Result>
class ArteRelationPointer: public ArtePointerBase
{
  typedef ArteRelationPointer<Relation,Result> self;
  typedef ArtePointer<Result> value_type;
  typedef value_type* pointer;
  typedef value_type& reference;

  friend class ArteRelVector<Relation>;
public:
  ArteRelationPointer(): ArtePointerBase( 0 ) {}
  ArteRelationPointer( const ArteRelationPointer<Relation,Result>& arg )
    : ArtePointerBase( arg ) {}
};


template<class Result> class ArteRelationIterator;


//
// first of three derived pointers
//
template<class Relation, class Result>
class ArteInvOneRelationPointer: public ArteRelationPointer<Relation,Result>
{
  typedef ArteInvOneRelationPointer<Relation,Result> self;
  typedef ArtePointer<Result> value_type;
  typedef value_type* pointer;
  typedef value_type& reference;
  typedef ArteRelationIterator<Result> iterator;
  typedef forward_iterator_tag iterator_category;
#ifndef __CINT__
  typedef ptrdiff_t difference_type;
#else
  typedef int difference_type;
#endif  

public:  
  iterator begin() const;
  iterator end() const;
  
  inline static ArteRelVector<Relation>& getLink();
#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
  inline static ArteRelVector<Relation>& getLinkLink()
    { return getLink(); }
#endif
};


//
// second of three derived pointers
//
template<class Relation, class Result>
class ArteManRelationPointer: public ArteRelationPointer<Relation,Result>
{
  typedef ArteManRelationPointer<Relation,Result> self;
  typedef ArtePointer<Result> value_type;
  typedef value_type* pointer;
  typedef value_type& reference;
  typedef ArteRelationIterator<Result> iterator;
  typedef forward_iterator_tag iterator_category;
#ifndef __CINT__
  typedef ptrdiff_t difference_type;
#else
  typedef int difference_type;
#endif  
public:
  iterator begin() const;
  iterator end() const;

  inline static class ArteRelVector<Relation>& getLink();
#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
  inline static ArteRelVector<Relation>& getLinkLink()
    { return getLink(); }
#endif
};

#ifdef USE_GATE
template<class Relation, class Result, class T>
ArteManRelationPointer<Relation,Result> T::* findArteManRelationEntry
( const ArteManRelationPointer<Relation,Result>& );
#endif


//
// third of three derived pointers
//
template<class Relation, class Result>
class ArteInvManRelationPointer: public ArteRelationPointer<Relation,Result>
{
  typedef ArteInvManRelationPointer<Relation,Result> self;
  typedef ArtePointer<Result> value_type;
  typedef value_type* pointer;
  typedef value_type& reference;
  typedef ArteRelationIterator<Result> iterator;
  typedef forward_iterator_tag iterator_category;
#ifndef __CINT__
  typedef ptrdiff_t difference_type;
#else
  typedef int difference_type;
#endif
public:
  iterator begin() const;
  iterator end() const;

  inline static class ArteRelVector<Relation>& getLink();
};


#ifdef USE_GATE
template<class Relation, class Result, class T>
ArteInvManRelationPointer<Relation,Result> T::* findArteInvManRelationEntry
( const ArteInvManRelationPointer<Relation,Result>& );
#endif


//
// ArteRelationIterator
//
template<class Result>
class ArteRelationIterator: public ArtePointerBase
{
  typedef ArteRelationIterator<Result> self;
  typedef ArtePointer<Result> value_type;
  typedef value_type* pointer;
  typedef value_type& reference;
  typedef self iterator;
  typedef forward_iterator_tag iterator_category;
#ifndef __CINT__
  typedef ptrdiff_t difference_type;
#else
  typedef int difference_type;
#endif  
  struct ArteOneToManyRelation
  {
    ArteOneToManyRelation(): next(0) {}
    ArtePointer<Result> el;
    ArtePointerBase next;
  };
  
  struct ArteManyToManyRelation
  {
    ArteManyToManyRelation(): next( ArtePointer<Result>( 0 )),
				    othernext(ArtePointer<Result>( 0 ))  {}
    ArtePointer<Result> el;
    ArtePointerBase next;
    int selfel;  // we do not know the type
    ArtePointerBase othernext;
  };

  struct ArteBackwardsManyToManyRelation
  {
    ArteBackwardsManyToManyRelation(): othernext(0), next(0) {}
    int selfel;  // we do not know the type
    ArtePointerBase othernext;
    ArtePointer<Result> el;
    ArtePointerBase next;
  };
  
public:
  ArteRelationIterator();

  template<class Relation>
  ArteRelationIterator( const ArteInvOneRelationPointer<Relation,Result>& x )
    : ArtePointerBase( x ),
      relation( &x.getLink() ),
      relType( 2 )
    {}
  
  template<class Relation>
  ArteRelationIterator( const ArteManRelationPointer<Relation,Result>& x )
    : ArtePointerBase( x ),
      relation( &x.getLink() ),
      relType( 3 )
    {}
  
  template<class Relation>
  ArteRelationIterator( const ArteInvManRelationPointer<Relation,Result>& x )
    : ArtePointerBase( x ),
      relation( &x.getLink() ),
      relType( 4 )
    {}
  
  template<class Relation>
  const ArteRelationIterator<Result>&
  operator=( const ArteInvOneRelationPointer<Relation,Result>& x )
    {
      *(ArtePointerBase*)this = x;
      relation = &x.getLink();
      reltype  = 2;
      return *this;
    }

  template<class Relation>
  const ArteRelationIterator<Result>&
  operator=( const ArteManRelationPointer<Relation,Result>& x )
    {
      *(ArtePointerBase*)this = x;
      relation = &x.getLink();
      reltype  = 3;
      return *this;
    }

  template<class Relation>
  const ArteRelationIterator<Result>&
  operator=( const ArteInvManRelationPointer<Relation,Result>& x )
    {
      *(ArtePointerBase*)this = x;
      relation = &x.getLink();
      reltype  = 4;
      return *this;
    }

  value_type operator*() const { return lookup(); }
  // Dereferences the pointer.

  operator value_type() const { return lookup(); }

  self& operator++() {
    followLink();
    return *this;
  }
  self operator++(int) {
    self tmp = *this;
    followLink();
    return tmp;
  }

  bool operator!=( const ArteRelationIterator<Result>& rhs )
    { return ArtePointerBase::operator!=( rhs ); }

#if !defined(ARTE_CXX_COMPAT_MODE) || (ARTE_CXX_COMPAT_MODE==1)
  int valid() const { return ArtePointerBase::operator!=(0); }
  void next() { followLink(); }
#endif
  
private:
  ArteManyToManyRelation getManyToMany() const
    {
#ifdef NO_ARTE
      return ArteManyToManyRelation();
#else
      void* p = gcbank_.iq + (*relation).getInt()+ 2;
      ArteManyToManyRelation* rp = (ArteManyToManyRelation*)p;
      return rp[index()-1];      
#endif
    }
  
  ArteBackwardsManyToManyRelation getBackwardsManyToMany() const
    {
#ifdef NO_ARTE
      return ArteBackwardsManyToManyRelation();
#else
      void* p = gcbank_.iq + (*relation).getInt() + 2;
      ArteBackwardsManyToManyRelation* rp
	= (ArteBackwardsManyToManyRelation*)p;
      return rp[index()-1];
#endif
    }
  
  ArteOneToManyRelation getOneToMany() const
    {
#ifdef NO_ARTE
      return ArteOneToManyRelation();
#else
      void* p = gcbank_.iq + (*relation).getInt() + 2;
      ArteOneToManyRelation* rp = (ArteOneToManyRelation*)p;
      return rp[index()-1];
#endif
    }

protected:
  ArtePointer<Result> lookup() const
    {
      switch( relType ) {
      case 2: { // OneToMany
	ArteOneToManyRelation rel = getOneToMany();
	return rel.el;
      }
      case 3: { // ManyToMany
	ArteManyToManyRelation rel = getManyToMany();
	return rel.el;
      }
      case 4:
      default:{ // BackwardsManyToMany
	ArteBackwardsManyToManyRelation rel = getBackwardsManyToMany();
	return rel.el;
      }
      }
    }
  void followLink()
    {
      if( index()!= 0 ) {
	switch( relType ) {
	  
	case 2: { // OneToMany
	  ArteOneToManyRelation rel = getOneToMany();
	  //indexReference() = rel.next.index();
	  ArtePointerBase::operator=( rel.next );
	  break;
	}
	case 3: { // ManyToMany
	  ArteManyToManyRelation rel = getManyToMany();
	  ArtePointerBase::operator=( rel.next );
	  break;
	}
	case 4:
	default: { // ManyToMany
	  ArteBackwardsManyToManyRelation rel
	    = getBackwardsManyToMany();
	  ArtePointerBase::operator=( rel.next );
	  break;
	}
	}
      }
    }    

private:
  ArteLink* relation;
  int relType;
};


template<class Relation, class Result>
inline ArteRelationIterator<Result>
ArteInvOneRelationPointer<Relation,Result>::begin() const
{
  return ArteRelationIterator<Result>( *this );
}


template<class Relation, class Result>
inline ArteRelationIterator<Result>
ArteManRelationPointer<Relation,Result>::begin() const
{
  return ArteRelationIterator<Result>( *this );
}


template<class Relation, class Result>
inline ArteRelationIterator<Result>
ArteInvManRelationPointer<Relation,Result>::begin() const
{
  return ArteRelationIterator<Result>( *this );
}


template<class Relation, class Result>
inline ArteRelationIterator<Result>
ArteInvManRelationPointer<Relation,Result>::end() const
{
  return ArteRelationIterator<Result>();
}


template<class Relation, class Result>
inline ArteRelationIterator<Result>
ArteManRelationPointer<Relation,Result>::end() const
{
  return ArteRelationIterator<Result>();
}


template<class Relation, class Result>
inline ArteRelationIterator<Result>
ArteInvOneRelationPointer<Relation,Result>::end() const
{
  return ArteRelationIterator<Result>();
}


// template<class Result>
// inline int operator==(const ArteRelationIterator<Result>& el1,
// 		      const ArteRelationIterator<Result>& el2)
// { return ArtePointerBase::operator==( el2 ); }


//
// ********************************************************************
//
//    createRelation
//
// ********************************************************************
//

//
// InvMan
//

template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T1> el1, ArtePointer<T2> el2,
		     ArteInvManRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  mmbrel_(  ArteInvManRelationPointer<Relation,T2>::getLink().getInt(),
	    el1, el2 );
#else
#endif
}


template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T2> el2, ArtePointer<T1> el1,
		     ArteInvManRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  mmbrel_(  ArteInvManRelationPointer<Relation,T2>::getLink().getInt(),
	    el1, el2 );
#else
#endif
}


//
// Man
//

#ifndef NO_ARTE
template<class Relation, class T1, class T2>
void createRelation(  ArtePointer<T1> el1, ArtePointer<T2> el2,
		      ArteManRelationPointer<Relation,T2> T1::* e
		      )
{
  mmbrel_(  ArteManRelationPointer<Relation,T2>::getLink().getInt(),
	    el2, el1 );
}
#else
template<class Relation, class T1, class T2>
void createRelation(  ArtePointer<T1> el1, ArtePointer<T2> el2,
		      ArteManRelationPointer<Relation,T2> T1::* e
		      )
{
  ArteManRelationPointer<Relation,T2>::getLink().setRelation( el1, el2 );
}
#endif


template<class Relation, class T1, class T2>
void createRelation(  ArtePointer<T2> el2, ArtePointer<T1> el1,
		      ArteManRelationPointer<Relation,T2> T1::* e
		      )
{
#ifndef NO_ARTE
  mmbrel_(  ArteManRelationPointer<Relation,T2>::getLink().getInt(),
	    el2, el1 );
#else
#endif
}


//
// InvOne
//

template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T1> el1, ArtePointer<T2> el2,
		     ArteInvOneRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  mmbrel_(  ArteInvOneRelationPointer<Relation,T2>::getLink().getInt(),
	    el1, el2 );
#else
#endif
}


template<class Relation, class T1, class T2>
void createRelation( ArtePointer<T2> el2, ArtePointer<T1> el1,
		     ArteInvOneRelationPointer<Relation,T2> T1::* = 0
		     )
{
#ifndef NO_ARTE
  mmbrel_(  ArteInvOneRelationPointer<Relation,T2>::getLink().getInt(),
	    el1, el2 );
#else
#endif
}


//
// inlines
//


#if defined(USE_GATE) && defined(USE_ROOT)
inline TBuffer& operator<<( TBuffer& buf, ArtePointerBase ap )
{
  return buf << ap.index();
}
    

inline TBuffer& operator>>( TBuffer& buf, ArtePointerBase& ap )
{
  int idx;
  buf >> idx;
  ap = idx;
  return buf;
}
    
    
template<class T1, class T2>
inline TBuffer& operator<<( TBuffer& buf, ArteRelationPointer<T1,T2> ap )
{
  return buf << ap.base;
}
    

template<class T1, class T2>
inline TBuffer& operator>>( TBuffer& buf, ArteRelationPointer<T1,T2>& ap )
{
  int idx;
  buf >> idx;
  ap.base = idx;
  return buf;
}
#endif /* USE_ROOT */
    
#endif /* _ARTE_TABLE_HH_ */
