//                              -*- Mode: C++ -*-
//: EAMCTruthAnalyzer.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 26/5/97
// Last Update ........
//

#include "care/EMClusterTree.hh"
#ifndef ONLINE
#include "care/EMHitCell.hh"
#include "care/EAMCTruthAnalyzer.hh"
#include "care/ECMCcalib.hh"

#ifdef _OLD_NAMES_
#  include <algo.h>
#else
#  include <algorithm>
#endif

#include <iostream.h>
#include"care/EFCompareDeposits.hh"
#undef  WRITE

/* 
CLASS
  EAMCTruthAnalyzer
  Class to find the monte carlo particle that contributes the most to
  the energy released in a cluster.

USAGE
  Call the cluster function analyzeMe() with an instance of this class as
  a parameter; then ask for the maximum energy deposit:
  EXAMPLE
    EMCluster  clust;
    // ... fill clust in some way
    
    EAMCTruthAnalyzer MCtruth;

    clust.analyzeMe(MCtruth);
    
    ECMCDeposit  dep = MCtruth.maxDeposit();

  END


};
*/

//////////////////// EAMCTruthAnalyzer Class Definitions ////////////////

// No functions
EAMCTruthAnalyzer::EAMCTruthAnalyzer() : depList(){}
EAMCTruthAnalyzer::~EAMCTruthAnalyzer() {}

//--
// Cluster Analyzer
void EAMCTruthAnalyzer::doAnalysis(const EMCluster * clus){
  // perform operation on all elements of the cluster
  for(const EMClusterMember* child = clus->begin(); 
      child!=clus->end();  child = child->next())
      child->analyzeMe(*this);
}

//--
// TrackOnEcal
void EAMCTruthAnalyzer::doAnalysis(const EMTrackOnEcal *){}

//--
// ClusterMember
void EAMCTruthAnalyzer::doAnalysis(const EMClusterMember *){}

//--
// ClusterTree
void EAMCTruthAnalyzer::doAnalysis(const EMClusterTree * tree){
  const EMCluster* cluster = tree;
  doAnalysis(cluster);
}

//--
// Hit Cell
void EAMCTruthAnalyzer::doAnalysis(const EMHitCell * hitT){
  // object for MC calibration
  ECMCcalib calibration;
  // get the HITC entry
  ArtePointer<HITC> hitc = hitT->whichBank();
  Energy_t cal = calibration[hitc];
  // now loop on relations of this cell with MCAL
  ArteRelationIterator<MCAL> relIter(hitc->mcal.begin());
  
  if( relIter != hitc->mcal.end() ){
    for( ; relIter != hitc->mcal.end() ; ++relIter ){
      // relIter now points to a MCAL entry
      ArtePointer<MCAL> mcal(*relIter);
      float deposit = mcal->de*cal;
      ArtePointer<MTRA> mtra =  mcal->mtra;
    
#ifdef WRITE
      cout<<" MC Track number "<<mtra.index()<<" Z begin:"
	  <<ArtePointer<MVER>(mtra->bver)->z<<" Bver:"<<mtra->btra;
#endif
      while(mtra->btra!=0 && ArtePointer<MVER>(mtra->bver)->z>1331.){
	mtra = mtra->btra;
#ifdef WRITE
	cout<<" .. Switch to : "<<mtra.index()<<" Z:"
	    <<ArtePointer<MVER>(mtra->bver)->z;
#endif
      }
#ifdef WRITE
      cout<<endl;
#endif
    // now mtra and deposit are defined
      ECMCDeposit depInfo( mtra, deposit);
      vector<ECMCDeposit>::iterator where = depList.begin(); 
      for(; where!=depList.end(); where++){
	if( (*where).track() == mtra) break;
      }
      if(  where!=depList.end()){
	// track already stored: add energy
	(*where) += depInfo;
      } else {
	// new track : add to list
	depList.push_back(depInfo);
      }
    }
  } 
}

void EAMCTruthAnalyzer::sortDeposits(){
  // Sort the enegy deposited
  sort(depList.begin(), depList.end(), EFCompareDeposits());
}

ECMCDeposit EAMCTruthAnalyzer::maxDeposit(){
  //
 if(depList.begin()!=depList.end() ){
    //cout<<".";
    return *depList.begin();
  } else {
    //    cout<<"0";
    return ECMCDeposit();
  }
}



//--
//  Return the second deposit after the maximum
ECMCDeposit EAMCTruthAnalyzer::second(){
  if(depList.begin()!=depList.end() && depList.begin()+1!=depList.end()){
    //cout<<".";
    return *(depList.begin()+1);
  } else {
    //    cout<<"0";
    return ECMCDeposit();
  }
}
//--
//  Return the second deposit after the maximum
void EAMCTruthAnalyzer::reset(){
  depList.erase(depList.begin(),depList.end());
}

#endif // ONLINE







