//                              -*- Mode: C++ -*-
//: ESCorCOGPos.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
// Last Update ........
//

#include "care/caredata.hh"
#include "care/EMClusterTree.hh"
#include "care/EMHitCell.hh"
#include <math.h>
#include "care/ESCorCOGPos.hh"
#include "care/ECCellID.hh"
#include "arte/ArteMessage.hh"

double arcsinhyp( double val);


/////////////////////// Static data initialization

bool  ESCorCOGPos::doit = true;
float  ESCorCOGPos::hardLamb[] = {0.,0.9932,1.403,0.,0.,4.075};
float  ESCorCOGPos::hardFreq[] = {0.,1.,1.,0.,0.,1.};
float  ESCorCOGPos::softLamb[] = {0.,0.,0.,0.,0.,0.};
float  ESCorCOGPos::softFreq[] = {0.,0.,0.,0.,0.,0.};
// Static parameters for errors
float  ESCorCOGPos::errorscal[] = {0.,65.,13.9,0.,0.,1.8};
float  ESCorCOGPos::errorcon[]  = {0.,-.9,-.23,0.,0.,-0.01};
float  ESCorCOGPos::ercorr[]    = {0.,2.1,2.1,0.,0.,8.0};


////////////////////// Static function definitions

//--
//  Filling the static data for corrections (light)
void  ESCorCOGPos::fill(SubCal subCal, float hardLam){
  fill(subCal,hardLam,1.,0.,0.);
}

//--
//  Filling the static data for corrections (full)
void ESCorCOGPos::fill(SubCal subCal, float hardLam, float hardFre,
                        float softLam, float softFre){
  static int ndivs [] = {0,5,2,1};
  int nd = ndivs[(int) subCal];
  //
  hardLamb[nd] = hardLam;
  hardFreq[nd] = hardFre;
  softLamb[nd] = softLam;
  softFreq[nd] = softFre;
}
 
//--
//  Set/reset flag
void ESCorCOGPos::setFlag(bool flag){
  doit = flag;
}


//--
//  Print informations on static data
void ESCorCOGPos::dump(){

  static int ndivs [] = {5,2,1};
  //                    01234567 89012345 67890
  static char nam[] = "Inner  \0Middle \0Outer  \0";

  arteMsg<<" Data for correction the extimation of the conversion point";
  arteMsg.allp();
  arteMsg<<" of a gamma on ECAL from the naive Center of Gravity.";
  arteMsg.allp();
  arteMsg<<" Correction flag: ";
  if(doit) arteMsg<<"ON";
  else arteMsg<<"OFF";
  arteMsg.allp();
  arteMsg<<" SubCal hard lambda  hard frequence   soft lambda  soft frequence";
  arteMsg.allp();
  for(int i=0; i<3; i++){
    int nd = ndivs[i];
    arteMsg<<nam[i*8]<<"  "<<hardLamb[nd]<<"  "<<hardFreq[nd]
	   <<"    "<<softLamb[nd]<<"  "<<softFreq[nd];
    arteMsg.allp();
  }

}

//////////////////// ESCorCOGPos Class Definitions ////////////////

ESCorCOGPos::ESCorCOGPos() : xn(0.0), yn(0.0) {}
ESCorCOGPos::~ESCorCOGPos(){}

//--
//  Start a cluster analysis
void ESCorCOGPos::evaluatePos(const EMCluster* clus){
  // get first the center of gravity
  ESCOGPos::evaluatePos(clus);
  //
  // Now the correction can be evaluated
  ESCorCOGPos::correct();
}

//--
// Correct on internal data 
void ESCorCOGPos::correct(){
  const EMHitCell* hitC = posData.cell();
  int incl = hitC->ncl();
  // Evaluate the new X and its error
  xn = apply( posData.x(), hitC->x(), incl);
  Coord_t sx = variance(xn, posData.x(), hitC->x(), 
	      posData.energy(), posData.sqenerr(), incl);
  // Evaluate the new Y and its error
  yn = apply( posData.y(), hitC->y(), incl);
  Coord_t sy = variance(yn, posData.y(), hitC->y(), 
	      posData.energy(), posData.sqenerr(), incl);
  // Store the data into the internal structures
  posData.setPoint(xn, yn, posData.z());
  posData.setVar( sx, 0.0, sy);
}

//--
//  Evaluate the correction on a given cog (by class Position)
void ESCorCOGPos::correct(const ESPosition & cog){
  posData = cog;
  correct();
}

//--
//  Evaluate the correction on a given cog (by class COGPos)
void ESCorCOGPos::correct(const ESCOGPos & cog){
  posData = cog.getData();
  correct();
} 

void ESCorCOGPos::applyTo(Coord_t xmean, Coord_t ymean, 
			  Coord_t xc, Coord_t yc, 
			  Energy_t energy, Energy_t envar,
			  int ndv, Coord_t varpos[3]) {
  xn = apply(xmean, xc, ndv);
  yn = apply(ymean, yc, ndv);
  varpos[0] = variance(xn, xmean, xc, energy, envar, ndv);
  varpos[1] = 0.0;
  varpos[2] = variance(yn, ymean, yc, energy, envar, ndv);
}

//--
//  apply the correction to a point (x,y) evaluated by the COG algorithm
//  and store the result in the body of the object. 
//  In input is required the pointer to the Hit cell in which
//  the COG is.
void ESCorCOGPos::applyTo(Coord_t xmean, Coord_t ymean,  // COG estimations
			  Energy_t energy, Energy_t envar, 
			  const EMHitCell* cell, Coord_t varpos[3] ){
  float xcc = cell->x();
  float ycc = cell->y();
  int ndivs = cell->ncl();
  applyTo(xmean, ymean, xcc, ycc, energy, envar, ndivs, varpos);
}

Coord_t ESCorCOGPos::apply(Coord_t mean, Coord_t center, int ndv) const {
  //
  // the correction applied is the inverse of the relation:
  // 
  //   Xb = W/2 * sinh(L*Xc)/ sinh(L*W/2)
  //
  // that holds for showers with a single biexponential lateral 
  // distribution shape. In the relations simbols have the following meanings:
  //
  //  W  is the width of the cell (cm)
  //  L  is the slope parameter of the lateral distribution (cm^-1)
  //  Xc is the true center of the shower (in the range (-W/2,W/2))
  //  Xb is the Center of gravity of the observed shower
  //

  static float fact[6] = {0.0,0.0,0.0,0.0,0.0,0.0};
  static bool initialized = false;
  if( !initialized){
    // inizialize the fact array
    int divs[3] = {1,2,5}; // the divisions of the cells considered
    float width = ECalInfo.xMsize;
    float w;
    for(int i=0; i<3; i++){
      int ndiv = divs[i];
      w = 0.5*width/((float) ndiv);
      fact[ndiv] = sinh(w*hardLamb[ndiv])/w;
    }
    initialized = true;
  }
  //
  Coord_t newextim = mean;
  if( doit ){ // test if corrections should be applyed
    float delta = mean-center;
    newextim =  arcsinhyp(delta*fact[ndv])/hardLamb[ndv]+center;
  }
  return newextim;
}

Coord_t ESCorCOGPos::variance(Coord_t /* corrected */, 
			      Coord_t mean, Coord_t center, 
			      Energy_t energy, Energy_t envar, 
			      int ndv ) const {
  //
  // The error on the corrected center of gravity is evaluated as
  // follows:
  //
  // Sigma_x(xcog, xcc, energy, sigma_e) = 
  //        = (A sigma_e/energy  (+) B ) / sqrt( ((xcog-xcc)*C)^2+1)
  //
  // where A, B, C are parameters dependent on the calorimeter type.
  //
  // Mind that the output of the present method is Sigma_x^2
  // so that the actual function used is:
  // Variance_x(xcog, xcc, energy, sigma_e) = 
  //        = (A variance_e/(energy*energy) + B ) / ( ((xcog-xcc)*C)^2+1)
  //

  float varx = 0.0;
  if( envar>0.000001 ){ // 1 MeV of error
    float z = (mean-center)*ercorr[ndv];
    z = 1./(z*z+1.0); // Correction to the error

    varx =  (errorscal[ndv]*sqrt(envar)/energy+errorcon[ndv])*z;
  }
  return varx;
}


// definition of the hyperbolic arc sin
double arcsinhyp( double val ){
  return log(val+sqrt(val*val+1.0));
}




