//----------------------------------------------------------------------------//
//
//   File:        EcalPretrigger.C
//
//   Description: simulation of the ECAL pretrigger
//
//   Author:      Michael Noerenberg
//                HERA-B Collaboration
//                DESY Hamburg
//                Notkestrasse 85
//                D-22603 Hamburg
//                Germany
//
//   e-mail:      mino@mail.desy.de
//
//----------------------------------------------------------------------------//

#include "care/EcalPretrigger.h"

#include <arte/arte.hh>

//----------------------------------------------------------------------------//

float EcalPretrigger::outerCellPitch;
float EcalPretrigger::middleCellPitch;
float EcalPretrigger::innerCellPitch;

float EcalPretrigger::cellPitch[3];

int   EcalPretrigger::numCellsX[3];
int   EcalPretrigger::numCellsY[3];

float EcalPretrigger::z_ECAL; // z-position of the ECAL //

float EcalPretrigger::m_Energie  [3][80+2][48+2];
float EcalPretrigger::m_Cluster  [3][80+2][48+2];

int   EcalPretrigger::m_MagnetOn;
float EcalPretrigger::m_Cut;

int   EcalPretrigger::m_GeoMode;
vector<EcalPretrigger::rectangle> EcalPretrigger::m_ActiveRects;
vector<int>                       EcalPretrigger::m_ActiveBoards;

//----------------------------------------------------------------------------//

int EcalPretrigger::readHITC()
{
  int hit_count = 0;

  reset();   // clear the cell and cluster memory //

  for(ArteTable<HITC>::iterator hitc = ArteTable<HITC>::begin();
      hitc != ArteTable<HITC>::end();
      ++hitc){

    switch (m_GeoMode) {

    case 0:   // accept all //

      addHit(hitc->q, hitc->x, hitc->y); break;

    case 1:   // consider defined rectangles //

      for (vector<rectangle>::iterator it = m_ActiveRects.begin();
	   it != m_ActiveRects.end();
	   ++it) {

	if (it->containsPoint(hitc->x, hitc->y))
	  addHit(hitc->q, hitc->x, hitc->y);
	
      }; break;

    case 2:   // get Readout board FED ID //

      ArtePointer<GCCE> gcce = hitc->gcce;
      ArtePointer<ECHS> echs(gcce.index());
      int fdid = echs->fdid;
      fdid &= 0xff; // use only board ID //

      for (vector<int>::iterator it = m_ActiveBoards.begin();
	   it != m_ActiveBoards.end();
	   ++it) {

	if (*it == fdid) {
	
	  addHit(hitc->q,
		 hitc->x,
		 hitc->y);

	}

      }; break;
    }
	

    ++hit_count;

  }

  return hit_count;
}

//----------------------------------------------------------------------------//

void EcalPretrigger::dumpRects()
{
  for(vector<rectangle>::iterator it = m_ActiveRects.begin();
      it != m_ActiveRects.end();
      ++it) {
    cout << it << ' '
	 << it->x_min << ' '
	 << it->x_max << ' '
	 << it->y_min << ' '
	 << it->y_max << ' ' << endl;
  }
}

//----------------------------------------------------------------------------//

void EcalPretrigger::setup()
{

  outerCellPitch  = 11.15;                // outer  cell pitch //
  middleCellPitch = outerCellPitch / 2.0; // middle cell pitch //
  innerCellPitch  = outerCellPitch / 5.0; // inner  cell pitch //

  cellPitch[0] = innerCellPitch;
  cellPitch[1] = middleCellPitch;
  cellPitch[2] = outerCellPitch;

  numCellsX[0] = numInnerCellsX;
  numCellsX[1] = numMiddleCellsX;
  numCellsX[2] = numOuterCellsX;

  numCellsY[0] = numInnerCellsY;
  numCellsY[1] = numMiddleCellsY;
  numCellsY[2] = numOuterCellsY;

  z_ECAL = 1350.0;  // z-position of the ECAL //

  m_MagnetOn = 0;   // default: magnet off     //
  m_Cut      = 1.0; // default: pt-cut 1.0 GeV //
  m_GeoMode  = 0;   // default: accept all     //

  for (int region = 0; region < 3; ++region)
    for (int x = 0; x < numCellsX [region] + 2; ++x)
      for (int y = 0; y < numCellsY [region] + 2; ++y) {
	m_Energie [region][x][y] = 0.0;
	m_Cluster [region][x][y] = 0.0;
      }

}

//----------------------------------------------------------------------------//

void EcalPretrigger::reset()
{

  for (int region = 0; region < 3; ++region)
    for (int x = 0; x < numCellsX [region] + 2; ++x)
      for (int y = 0; y < numCellsY [region] + 2; ++y) {
	m_Energie [region][x][y] = 0.0;
	m_Cluster [region][x][y] = 0.0;
      }

}

//----------------------------------------------------------------------------//

void EcalPretrigger::dumpCells(int region,
			       ostream& os)
{
  switch (region) {
  case 0: os << "------- inner  ECAL "; break;
  case 1: os << "------- middle ECAL "; break;
  case 2: os << "------- outer  ECAL "; break;
  default: return;
  }

  for (int x = numCellsX [region] + 1 - 18; x >= 0; --x)
    os << '-';
  os << endl;

  for (int y = numCellsY [region] + 1; y >= 0; --y) {
    os << '|';
    for (int x = numCellsX [region] + 1; x >= 0; --x) {
      if (m_Cluster [region][x][y] > 0.001)
	os << '#';
      else {
	if (m_Energie [region][x][y] < 0.001)
	  os << ' ';
	else
	  os << '+';
      }
    }
    os << '|' << endl;
  }
  
  for (int x = numCellsX [region] + 1; x >= 0; --x)
    os << '-';
  os << "--" << endl;
}

//----------------------------------------------------------------------------//

void EcalPretrigger::addHit(float energie,
			    float x_pos,
			    float y_pos)
{

  int region, x_cell, y_cell;

  getCell(x_pos, y_pos, region, x_cell, y_cell);

  m_Energie [region][x_cell][y_cell] = energie;

}

//----------------------------------------------------------------------------//

int EcalPretrigger::searchTowers()
{
  int tower_counter = 0;

  for (int region = 0; region < 3; ++region)
    for (int x = 1; x < numCellsX [region] + 1; ++ x)
      for (int y = 1; y < numCellsY [region] + 1; ++ y)
	{
	  if (testCut(m_Energie[region][x][y], region, x, y)) {
	    
	    m_Cluster[region][x][y] = m_Energie[region][x][y];
	    ++tower_counter;

	  } else
	    m_Cluster[region][x][y] = 0.0;
	}

  return tower_counter;
}

//----------------------------------------------------------------------------//

int EcalPretrigger::processTowers()
{
  readHITC();
  return searchTowers();
}

//----------------------------------------------------------------------------//

int EcalPretrigger::searchClusters()
{
  int cluster_counter = 0;

  for (int region = 0; region < 3; ++region)
    for (int x = 1; x < numCellsX [region] + 1; ++ x)
      for (int y = 1; y < numCellsY [region] + 1; ++ y)
	{
	  if (testCut(2.0 * m_Energie[region][x][y], region, x, y)) {

	    float sumEnergie = m_Energie[region][x][y];
	    sumEnergie += m_Energie[region][x+1][  y];
	    sumEnergie += m_Energie[region][x+1][y-1];
	    sumEnergie += m_Energie[region][  x][y-1];
	    sumEnergie += m_Energie[region][x-1][y-1];
	    sumEnergie += m_Energie[region][x-1][  y];
	    sumEnergie += m_Energie[region][x-1][y+1];
	    sumEnergie += m_Energie[region][  x][y+1];
	    sumEnergie += m_Energie[region][x+1][y+1];
	    if (testCut(sumEnergie, region, x, y)) {
	      
	      m_Cluster[region][x][y] = m_Energie[region][x][y];
	      ++cluster_counter;
	    
	    } else
	      m_Cluster[region][x][y] = 0.0;
	  } else
	    m_Cluster[region][x][y] = 0.0;
	}

  return cluster_counter;
}

//----------------------------------------------------------------------------//

int EcalPretrigger::processClusters()
{
  readHITC();
  return searchClusters();
}

//----------------------------------------------------------------------------//

int EcalPretrigger::getRegion(float x_pos,
			      float y_pos)
{

  if ((fabs(x_pos) < 3 * outerCellPitch && fabs(y_pos) < 4 * outerCellPitch) ||
      (fabs(x_pos) < 5 * outerCellPitch && fabs(y_pos) < 3 * outerCellPitch) ||
      (fabs(x_pos) < 7 * outerCellPitch && fabs(y_pos) < 2 * outerCellPitch)) {

    return 0;   // inner region //

  } else if ((fabs(x_pos) <  4 * outerCellPitch && fabs(y_pos) < 12 * outerCellPitch) ||
	     (fabs(x_pos) < 12 * outerCellPitch && fabs(y_pos) <  9 * outerCellPitch) ||
	     (fabs(x_pos) < 16 * outerCellPitch && fabs(y_pos) <  6 * outerCellPitch) ||
	     (fabs(x_pos) < 20 * outerCellPitch && fabs(y_pos) <  3 * outerCellPitch)) {

    return 1;   // middle region //

  } else {

    return 2;   // outer region //

  }

} 

//----------------------------------------------------------------------------//

void EcalPretrigger::getCell(float x_pos,
			     float y_pos,
			     int&  region,
			     int&  x_cell,
			     int&  y_cell)
{

  region = getRegion(x_pos, y_pos);

  x_cell =
    (int) ((cellPitch[region] * (float) numCellsX[region] / 2.0 + x_pos) / cellPitch[region]) + 1;
  y_cell =
    (int) ((cellPitch[region] * (float) numCellsY[region] / 2.0 + y_pos) / cellPitch[region]) + 1;

}

//----------------------------------------------------------------------------//

void EcalPretrigger::getPosition(int    region,
				 int    x_cell,
				 int    y_cell,
				 float& x_pos,
				 float& y_pos)
{
  
  x_pos = cellPitch[region] * ((float) x_cell - 1 - (float) numCellsX[region] / 2.0 + 0.5);
  y_pos = cellPitch[region] * ((float) y_cell - 1 - (float) numCellsY[region] / 2.0 + 0.5);
  
}

//----------------------------------------------------------------------------//

int EcalPretrigger::testGeo(float x_pos,
			    float y_pos)
{

  switch (m_GeoMode) {
  case 0:     // ignore geometry //
    return 1;
    break;
  case 1:     // look at defined rectangles //
    for (vector<rectangle>::iterator it = m_ActiveRects.begin();
	 it != m_ActiveRects.end();
	 ++it) {

      if (it->containsPoint(x_pos, y_pos)) return 1;

    }
    break;
  }
  return 0;
}

//----------------------------------------------------------------------------//

int EcalPretrigger::testCut(float energie,
			    float x_pos,
			    float y_pos)
{
  if (m_MagnetOn) { // magnet on -> apply Ktrig-cut //
    
    if (fabs(x_pos) > outerCellPitch &&
	fabs(y_pos) > outerCellPitch) { // avoid division by zero //

      float x2y2 = sqrt(x_pos*x_pos + y_pos*y_pos);
      float x2y3 = sqrt(x_pos*x_pos + fabs(y_pos*y_pos*y_pos));

      return (energie > m_Cut * (1/x2y2 + 1/x2y3) ?   // E > K_trig * ... //
	      1 : 0);

    } else
      return 0;

  } else {          // magnet off -> apply pt-cut //

    float x2y2   = sqrt(x_pos*x_pos + y_pos*y_pos);
    float x2y2z2 = sqrt(x_pos*x_pos + y_pos*y_pos + z_ECAL*z_ECAL);

    return (energie * x2y2 / x2y2z2 > m_Cut ?   // pt > m_Cut //
	    1 : 0);

  }
}

//----------------------------------------------------------------------------//

int EcalPretrigger::testCut(float energie,
			    int   region,
			    int   x_cell,
			    int   y_cell)
{
  float x_pos, y_pos;
  getPosition(region, x_cell, y_cell, x_pos, y_pos);
  return testCut(energie, x_pos, y_pos);
}

//----------------------------------------------------------------------------//










