//                              -*- Mode: C++ -*-
//: EMHitCell.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 3/4/97
// Last Update ........
//

#ifndef EMHITCELL_HH_
#define EMHITCELL_HH_

#include "care/EMClusterMember.hh"
#include "care/ECALtypes.hh"

/**
class EMHitCell.
  Concrete derived class for the hierarchical cluster tree used in
  pattern recognition of ECAL. This class is a "leaf": it does not have
  children, it cannot receive Add, Move or Remove message.

*/

class EMHitCell : public EMClusterMember {
public:
  /**@name Constructors and Destructor */
  //@{
  /** Constructors.
     The construction of EMHitCell is done referring to some
     parent. The depth level is adjusted accordingly. */
  EMHitCell( EMCluster* parent, 
	    ArtePointer<HITC> currCell);
  ///  The default constructor should be used only by the HCBuilder class.
  EMHitCell();
  /// Destructor: a virtual destructor is needed
  virtual ~EMHitCell();

  //@}

  /**@name Status access functions */
  //@{
  /// Return a flag to tell that this EMClusterMember IS a leaf.
  virtual bool isALeaf() const;
  //--
  /// Return the maximum Energy of the Cell itself. 
  virtual Energy_t  maxEnergy() const;

  //--
  ///  Return pointer to HITC bank.
  inline ArtePointer<HITC> whichBank() const;

  //--
  ///  Dump for debugging.
  virtual void dump() const;
  //@}

  //-- 
  ///  set internal pointer to hit cell
  void pointToCell(ArtePointer<HITC> currCell);

  //--
  /** Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
   that are object with definite analyzing properties. 
   A direct call to the doAnalysis() member of EAClusterAnalyzer for this
   "leaf" is done. */
  virtual void analyzeMe(EAClusterAnalyzer & analyzer) const;

  //--
  /** Analysis and traversing of the cluster tree is done by EAClusterAnalyzers
   that are object with definite analyzing properties. 
   A direct call to the doModify() member of EDClusterModifier for this
   "leaf" is done. */
  virtual void modifyMe(EDClusterModifier & modifier);
  //--
  /// Access the x coordinate of the center of a hit cell
  inline Coord_t  x() const;
  //--
  /// Access the y coordinate of the center of a hit cell
  inline Coord_t  y() const;

  //--
  /// Access the z coordinate of the front of a hit cell
  Coord_t  z() const;

  //--
  /// Access the energy of a hit cell
  virtual Energy_t  energy() const;

  //--
  /// Access the squared energy error of a hit cell
  virtual Energy_t  sqerror() const;

  //--
  /// Access the type of a hit cell
  int  ncl() const;

  //--
  /// Access the index of a hit cell
  CellIdx_t  cellIdx() const;
  
  friend class EMCluster;
 
protected:

private:
  /**@name Private Data */
  //@{
  /** 
    This is a link to a HITC entry in the Arte banks referring to a
    Hit Cell. */ 
  ArtePointer<HITC>  cell;
  //@}
};

//typedef EMHitCell HitCell;

#include "care/EMHitCell.icc"
#endif // EMHITCELL_HH_




