
// *****************************************************************************
//
// source:    faEmg.C
//
// created:   30 Jul 97
// updated:    1 Jun 99 Andreas Gellrich
//
// Author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   removed emg status flag (AG: 12 Jan 99)
//            polishing (AG: 14 Jan 99)
//            polishing (AG: 2 Feb 99)
//            no READALL in emg for cycle mode (AG: 22 Feb 99)
//            correct emg_consum() and emg_get_event() (AG: 25 Mar 99)
//            put includes (AG: 1 Jun 99)
//
// *****************************************************************************

// includes
#include <iostream>

#include "emg.h"
#include "grr.h"

#include "farm/fa.hh"

// externals
extern int  CYCLE;     // cycle mode
extern char *PARNAME;  // emg partition name;
extern int  SEG_NUM;   // emg partitions
extern int  SEG_LEN;   // emg segment size

extern char *evtBuf;   // event buffer

// *****************************************************************************
//
// name:      faReadEmg
//
// purpose:   read event from emg
//
// called by: faReadEmg
//
// input:     void
// return:    int  flag
//
// *****************************************************************************

int
faReadEmg()
{
  // pre-sets
  int        rc    = GRR_OK;
  static int count = 0;
  
  // print
  if( count < 10 ){
#ifdef _OUT_
    cout << "EMG: Waiting for event ....." << endl;
#endif
  }

  //
  // get event
  //
  if( ( rc = emg_get_event( (int *) evtBuf, SEG_LEN, 0 ) ) == GRR_OK ){
    if( count < 10 ){
      count++;
#ifdef _OUT_
      cout << "EMG: Got event" << endl;
#endif
    }
    return rc;
  }
  else {
#ifdef _ERR_
    cerr << "EMG: ERROR: getting event rc=" << rc << endl;
#endif
    return rc; // don't come back
  }
}

// *****************************************************************************
//
// name:      faInitEmg
//
// purpose:   initialize emg
//
// called by: fainit_
//
// input:     void
// return:    int  error code
//
// *****************************************************************************

int
faInitEmg()
{
  //
  // emg_consum() flag
  //
  int flag = READALL;
  if( CYCLE ){
    flag = 0;
  }

  //
  // init emg
  //
  int rc = GRR_OK;
  if( ( rc = emg_init() ) != GRR_OK ){ // initialize emg
#ifdef _OUT_
    cerr << "EMG: ERROR: init rc=" << rc << endl;
#endif
    return rc;
  }

  //
  // emg ctrl
  //
#ifdef _OUT_
  cout << "EMG: "
       << "partition="   << PARNAME << " "
       << "segments="    << SEG_NUM << " "
       << "length="      << SEG_LEN << " "
       << "flag="        << flag    << endl;
#endif

  //
  // SEG_NUM
  //
  if( ( rc = emg_ctrl( NSEGS, &SEG_NUM ) ) != GRR_OK ){ // set parameters
#ifdef _OUT_
    cerr << "EMG: ERROR: segments rc=" << rc << endl;
#endif
    return rc;
  }

  //
  // SEG_LEN
  //
  if( ( rc = emg_ctrl( SEGLEN, &SEG_LEN ) ) != GRR_OK ){
#ifdef _OUT_
    cerr << "EMG: ERROR: length rc=" << rc << endl;
#endif
    return rc;
  }

  //
  // attach to emg
  //
  rc = emg_attach( PARNAME, 0 ); // create new emg
  
  if( rc == GRR_FISHY ){
#ifdef _OUT_
    cerr << "EMG: ERROR: (GRR_FISHY) partition NOT built." << endl;
#endif
    return rc;
  }
  else if( rc == GRR_UNKNOWN ){
#ifdef _OUT_
    cerr << "EMG: ERROR: (GRR_UNKNOWN) partition NOT found." << endl;
#endif
    return rc;
  }
  else if( rc == GRR_INCOMPATIBLE ){
#ifdef _OUT_
    cerr << "EMG: ERROR: (GRR_INCOMPATIBLE) partition USED somewhere else?" << endl;
#endif
    return rc;
  }
  else if( rc != GRR_OK ){
#ifdef _OUT_
    cerr << "EMG: ERROR: rc=" << rc << endl;
#endif
    return rc;
  }

  //
  // consum
  //
  if( ( rc = emg_consum( -1, -1, 1, flag ) ) != GRR_OK ){ // consum emg
#ifdef _OUT_
    cerr << "EMG: ERROR: consuming "
	 << "flag=" << flag << " "
	 << "rc="   << rc   << endl;
#endif
    return rc; // error
  }
  
  return rc; // okay
}

// *****************************************************************************
//
// name:      faStopEmg
//
// purpose:   terminate
//
// called by: Fortran program ATSTOP
//
// input:     void
// return:    int  error code
//                 =0: ok
//                 <0: error
//
// *****************************************************************************

int
faStopEmg()
{
  int rc;
  if( ( rc = emg_release( PARNAME, 0 ) ) == GRR_OK ){ // realease emg
#ifdef _OUT_
    cout << "EMG: released." << endl;
#endif
    return rc;
  }
  else {
#ifdef _ERR_
    cerr << "EMG: ERROR: releasing rc=" << rc << endl;
#endif
    return rc;
  }
}
