
// *****************************************************************************
//
// source:    faInit.C
//
// type:      source code
//
// created:    7 Jul 97
// updated:    1 Jun 99 Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    fainit_()
//
// input:     void
// return:    int  ARTE_error_code
//
// called by: ATINIT
//
// purporse:  initialization of farm stuff called for FARM mode only
//
// changes:   initialize and register rpm (AG: 25-Nov-98)
//            multiple file input (AG: 27-Nov-98)
//            daq format logging (AG: 1-Dec-98)
//            includes (AG: 11 Jan 99)
//            removed emg status flag (AG: 12 Jan 99)
//            obey ANSI rules (AG: 15 Jan 99)
//            call logger initialization at the end (AG: 21 Jan 99)
//            rpm register (AG: 2 Feb 99)
//            some printing removed (AG: 10 Feb 99)
//            removed faUtil.hh (AG: 23 Feb 99)
//            use grr for error logging (AG: 25 Feb 99)
//            polishing (AG: 11 Mar 99)
//            put includes (AG: 1 Jun 99)
//
// *****************************************************************************

// includes
#include <iostream>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>

#include "rpm.h"
#include "grr.h"
#include "log.h"

#include "farm/fa.hh"

// globals
char *evtBuf = 0;  // event buffer
int nodeID   = -1; // default 4LT node ID

// externals
extern int  nArg;      // line command arguments
extern char **vArg;    // line command arguments

extern int  FARM;      // FARM mode
extern int  FAON;      // FARM online mode
extern char *DBNAME;   // database name
extern int  NOSPAR;    // sparsification
extern int  RPMINIT;   // init rpm
extern int  DAQLOG;    // daq format logging
extern int  FAGRR;     // error logger
extern char *PARNAME;  // emg partition
extern int  SEG_NUM;   // emg partitions
extern int  SEG_LEN;   // emg partition size

// C++ prootypes
void faCatchSIGINT ( int );
void faCatchSIGQUIT( int );
void faCatchSIGTERM( int );

// C prototypes
#ifdef sun
extern "C" int  gethostname( char *hostname, int length );
extern "C" long gethostid();
#endif

//
// program code
//
extern "C" int
fainit_()
{
  //
  // rpm
  //
  int rc = GRR_OK;
  if( RPMINIT ){
    // rpm init
    if( ( rc = rpm_init( nArg, (const char **) vArg ) ) != GRR_OK ){
#ifdef _ERR_
      cerr << "INIT: ERROR: rpm_init() rc=" << rc << endl;
#endif
      if( FAGRR ){
	  grr_error( GRR_SFATAL, "INIT: Arte rpm_init() FAILED.", rc );
      }
      return ARTE_ERR;
    }
    else {
#ifdef _OUT_
      cout << "INIT: rpm_init() rc=" << rc << endl; 
#endif
    }
    
    // rpm open
    if( ( rc = rpm_open( RPM_REGISTER ) ) != GRR_OK ){
#ifdef _ERR_
      cerr << "INIT: ERROR: rpm_open() rc=" << rc << endl;
#endif
      if( FAGRR ){
	  grr_error( GRR_SFATAL, "INIT: Arte rpm_open() FAILED.", rc );
      }
      return ARTE_ERR;
    }
    else {
#ifdef _OUT_
      cout << "INIT: rpm_register() rc=" << rc << endl;
#endif
    }
  }

  //
  // farm mode
  //
  if( FARM ){
    
    // 
    // init signal catchers
    //
    signal( SIGINT,  faCatchSIGINT  );
    signal( SIGQUIT, faCatchSIGQUIT );
    signal( SIGTERM, faCatchSIGTERM );
    
    //
    // node ID
    //
    unsigned int nodeID = gethostid();
#ifdef linux
    nodeID = ( nodeID & 0x0000ff00 ) >> 8;
#else
    nodeID = ( nodeID & 0x000000ff );
#endif

    //
    // init buffer
    //
    if( ( evtBuf = new char[SEG_LEN] ) != 0 ){
#ifdef _OUT_
      cout << "INIT: event buffer size=" << SEG_LEN << endl;
#endif
    }
    else {
#ifdef _ERR_
      cerr << "INIT: ERROR: event buffer" << endl;
#endif
      return ARTE_ERR;
    }
    
    //
    // online running
    //
    if( FAON ){
      
      // init emg
      if( ( rc = faInitEmg() ) != GRR_OK ){
	if( FAGRR ){
	  grr_error( GRR_SFATAL, "INIT: Arte emg FAILED.", rc );
	}
	return ARTE_ERR;
      }
    }
  }

  //  
  // init logging (definition of data type via line command)
  //
  if( DAQLOG && RPMINIT ){
    while( 1 ){
      if( ( rc = l4aLogInit( nArg, (const char **) vArg ) ) == GRR_OK ){
#ifdef _OUT_
	cout << "INIT: l4aLogInit() rc=" << rc << endl;
#endif
	break;
      }
      // error init logger
      else {
#ifdef _ERR_
	cerr << "INIT: ERROR: l4aLogInit() rc=" << rc << " Trying again ..." << endl;
#endif
	if( FAGRR ){
	  grr_error( GRR_SERROR, "INIT: Arte l4aLogInit() RETRY.", rc );
	}
	sleep( 5 );
      }
    }
  }

  // grr error message
  if( FAGRR ){
    grr_error( GRR_SINFO, "INIT: Arte faInit() DONE.", 0 );
  }
  
  // return
  return ARTE_GOOD;
}
