
/***********************************************************************
*-- Author :    Siegmund Nowak
*
*  This is the CERN user routine called at the end of each tracking step
*  Here actions are collected which are specific for HERA-B but general
*  for all HERA-B users. The individual user routine is called USSTEP 
*  The actions called from GUSTEP are:
*  - drawing of tracks
*  - filling of temporary MIMP/MCAL banks for sensitive detectors
*  - storing of new generated particles, restoring of STAK particles 
*  - calling of USSTEP
*  - limiting of spiraling particles in the magnet
*  - setting length and flag parameters for ending tracks
***********************************************************************/

#include <math.h>
#include <vector.h>
#include <stdio.h>
#include <ctype.h>
#include "gean/gckine.h"
#include "gean/gcking.h"
#include "gean/gcflag.h"
#include "gean/gctrak.h"
#include "gean/gcvolu.h"
#include "gean/gcsets.h"
#include "gean/ZGMOTH.h"
#include "gean/ZGESL.h"
#include "gean/ZGDETV.h"
#include "gean/zgkine.hh"
#include "gean/ZGTRDE.h"
#include "gean/ZGCAFM.h"
#include "gean/ZGTURB.h"
#include "arte/arte.hh"
#include "arte/ArteFlag.h"


extern "C" {
   void gustep_();               
   void mustcheck_(char*,int*);
   void gsxyz_();                
   void gdcxyz_();
   void crhibk_(int);            
   void trade_();
   void gsking_(int *);          
   void ghstak_();
   void muturb_();               
   void usstep_();
   void nspirl_(); 
   void cafmbk_(); 
}

void gustep_() {
   extern vector<hgmtra> *hg_mtra;
   union  { int ii; char cname[4];};

// define some useful quantities
// select volumina where particles are only in stack (jspec!= )  
   const int maxspec = 9;   
   char idnam[maxspec*4+1]; 
   strcpy(idnam,"CAY1CAY2CAY3CASCCAPBCATUSCRPPOLSPOLC");
//               ^   ^   ^   ^   ^   ^   ^   ^   ^
   ii = gcvolu_.names[gcvolu_.nlevel-1];
   for(int i =0; i<4; i++) cname[i] = toupper(cname[i]);  
   int jspec = 0;    int icheck = 0;
   for(int i=0; i < 4*maxspec; i+=4){
      if (!strncmp(cname,&idnam[i],4)) {
         jspec = i/4 + 1;  break; 
      }
   }  
// speciality for muon system
   char mname[5]; 
   strncpy(mname,cname,4);
   mname[4] = '\0';
   mustcheck_(mname,&icheck);
   if(icheck != 0) jspec =maxspec+1;

// get some track valuestrack values
   float  abs_charge = fabs(gckine_.charge); 
   vector<hgmtra>::iterator mt = hg_mtra->begin() + gckine_.itra -1;

// draw tracks if wanted
   if(!arteflag_.ibatch && gcflag_.iswit[0] && abs_charge >= 1.e-2) {
       gsxyz_();   gdcxyz_();
   }


   int istpu = 0;
   if(jspec > 0 && jspec <= 6) { 
//    make a calorimeter endpoint mtcp  at the entry of the calorimeter 
      if(!gckine_.istak && !mt->ep ) { 
         crhibk_(5);
         if(!strncmp(zgcafm_.CCAFM,"ON",2)) { 
            ArtePointer<GESL> gsl(zgmoth_.CALOSL);
            if(gckine_.itrtyp <= 2 && gctrak_.getot >= 4. &&
               (gctrak_.vect[2]-zgmoth_.ZZERO-gsl->z1) <= 11.) {
//             fast MC for electron or photon with energy > 4 GeV
//             and Z coordinate < z(begin calorimeter) + unsensitive step
               cafmbk_();  goto L60; 
            } 
         } 
      }
   }    

   else if(!strncmp(cname,"KILL",4)) {  
//      stop the particle if it enters in volume "kill"     
        gctrak_.istop=1;     goto L60;
   }

   else if(!strncmp(cname,"TR",2) || gckine_.ipart == 48 ) { 
//      call special code for trd-simulation
        if(trdsw_.itrswt == 1) trade_(); 
   }

   else if(!strncmp(cname,"MU",2)) {
//      switch on fast electron tracing by means of mututb
        if(int(arteflag_.geomvers) != 97 && turb_.TURBFL == 1) muturb_();
   }

   else if(abs_charge >= .1 && !mt->nhit){
//      stop tracks to avoid spirals in magnetic field
        nspirl_();
        if(gctrak_.istop && !gckine_.istak && arteflag_.mimpsw) istpu = -1; 
   }


// mimps will be written out
   if(arteflag_.mimpsw) {
      if(!gcsets_.idtype) {}
      else if(gcsets_.idtype == LGESL_SIL && zgdetv_.DETV[LGESL_SIL-1] == 'S'){ 
             if(gctrak_.inwvol == 1 && abs_charge > 1.e-2)  crhibk_(6);
      }   
      else if(gcsets_.idtype == LGESL_MSG && zgdetv_.DETV[LGESL_MSG-1] == 'S'){
             if(gctrak_.inwvol == 1 && abs_charge > 1.e-2) crhibk_(6);
      }   
      else if(gcsets_.idtype == LGESL_ECA) {
             if(gctrak_.destep >0 ) crhibk_(2);
      } 
      else if(gcsets_.idtype == LGESL_HOD1 || gcsets_.idtype == LGESL_HOD2 
             || gcsets_.idtype == LGESL_HOD3) {
             if(gctrak_.destep>0) crhibk_(3);
      }
      else if(gcsets_.idtype == LGESL_TRD && trdsw_.itrswt == 1){
             if(abs_charge > 1.e-2 || gckine_.ipart == 48) crhibk_(4);
      }
      else if(gcsets_.idtype == LGESL_RIC) { 
             if(gctrak_.inwvol == 1 && abs_charge > 1.e-2 )  crhibk_(1);
      }
      else { if(gctrak_.inwvol == 1 && abs_charge > 1.e-2 )  crhibk_(1);
      }
   }
      
// store newly generated particles in jkine/jvertx if they are outside
// the volumes idnam and in stak only if they are inside
   if(gcking_.ngkine > 0) {
      int ifl0 = 1;   if(jspec != 0) ifl0 = 0; 
      for (int i=0; i<gcking_.ngkine; i++)  gcking_.iflgk[i] = ifl0;  
      int i0 = 0;     gsking_(&i0);  
   }  

// restore stak particles in jkine/jvertx banks again, if they come
// out of volumes idnam
   if(gckine_.istak != 0 && gctrak_.inwvol == 1) {
      if(!strncmp(cname,"TR",2) || gckine_.ipart == 48) {}
      else if(jspec == 0) { 
         ghstak_();  gctrak_.istop = 1; 
      } 
   }



L60:   // call user subroutine
    if(gctrak_.istop && !istpu ) istpu = 1;
    usstep_();


    if(gctrak_.istop) {               // stopping tracks: give length and flag  
       if(!istpu) istpu = 2;

       if(!gckine_.istak && arteflag_.mimpsw) {
          mt->len = gctrak_.sleng;

          if(gctrak_.inwvol == 3) {                  // leaving the detector  
             mt->flag = 5;   return; 
          }
          if(istpu < 0) {                       // spiraling cut
	     mt->flag = 7;   return;
          }
          if(istpu == 2){                       // user stop  
             mt->flag = 8;   return;
          }
          if(!strncmp(cname,"KILL",4)) {        // volume KILL
             mt->flag = 5;    return; 
          }

          for(int i=0; i<gctrak_.nmec; i++) {

             if(gctrak_.lmec[i] == 5) {         // decay
                mt->flag = 3;   return; 
             }

             if((gctrak_.lmec[i] >= 6 && gctrak_.lmec[i]<= 8) 
                   ||   gctrak_.lmec[i] ==11 || gctrak_.lmec[i]== 12) {
                mt->flag = 4;   return;         //  interaction
              }

             if(gctrak_.lmec[i] == 30) {        // energy cutoff
                 mt->flag = 6;   return; 
             }
          }
       }
    }
    
    return;

       
}



