/* This file defines DetectorMenu class.
   Currently it contains 
     SetupMenu<HPT_Setup> hpt;

   Thus one may access, e.g., the "chamber" menu of High-Pt setup via:
     detector_menu.hpt.chamber.

   If you have function object "f" so that expression
     f(&is, ichamber);
   is valid ("is" -> istream, ichamber ->HPT_Setup::IChamber),
   you may construct the interactive command with 1 parameter and add
   it to the "chamber" menu:
     detector_menu.hpt.chamber("command_name") =
       make_interactive_command_1par(<you_function_object>, 
       (HPT_Setup::IChamber*)0);

   To implement the Setups with other then High-Pt types one should:
   1) add the corresponding SetupMenu<New_Setup>& new_setup;
   in the SetupMenuCaller class;
   2) change SetupMenuCaller constructor so that it accepts 
   additional argument SetupMenu<New_Setup>& new_menu and stores this
   reference in "new_setup" field;
   3) add 
     void operator()(New_Setup& setup) { new_setup.setup(p_is, &setup); }
   operator analogous to the High-Pt case
     void operator()(HPT_Setup& setup) { hpt.setup(p_is, &setup); }
   written below;
   
   4) add SetupMenu<New_Setup> new_setup; in the DetectorMenu class
   (note that in SetupMenuCaller only the reference  was added);
   5) add the line
        new_setup = menu.new_setup;
      like 
        hpt = menu.hpt;
      in void DetectorMenu::assign(const DetectorMenu& menu);
   6) change the call to the constructor of SetupMenuCaller in
        void DetectorMenu::initialize_sub_menus() {
	  (*this)("sub-detector") = make_interactive_command
	    (bind2(SetupMenuCaller(hpt), (istream**)0, p_det));
                   -------------------
      according to new arguments the constructor accepts (see 2)) 
*/   

// Author V.Balagura (balagura@mail.desy.de)   (28.02.99).

#ifndef _detector_menu_hh
#define _detector_menu_hh 1

#include "ptset/setup_menu.hh"
#include "ptset/detector_tools.hh"
#include "ptset/istream_menu.hh"
#include "ptset/call_interactively.hh"

class SetupMenuCaller {
  istream* p_is;

  SetupMenu<HPT_Setup>& hpt;
public:
  SetupMenuCaller(SetupMenu<HPT_Setup>& hpt_menu) : hpt(hpt_menu) {}
  //  virtual
  void operator()(istream* is, Detector* p_detector) {
    string setup_name;
    if ( !(*is >> setup_name) ) { 
      global_error()("can not read setup name");
      return;
    }
    p_is = is;
    apply_virtually(*this)(*p_detector, setup_name);
  }

  void operator()(HPT_Setup& setup) { hpt.setup(p_is, &setup); }
};
inline void create_detector_setup
(const string& setup_type, const string& setup_name, Detector* p_detector) {
  if ( setup_type == "HPT" ) {
    (*p_detector)[setup_name] = new HPT_Setup();
  }
  else {
    global_error()("setup type " + setup_type + " not defined");
    return;
  }
}


class DetectorMenu : public IstreamMenu {
  Detector* p_det;
public:
  SetupMenu<HPT_Setup> hpt;

  DetectorMenu(Detector* p_detector) : p_det(p_detector) {
    parser = new SlashBlock_InteractiveParser;
    initialize_sub_menus();
    (*this)("create") = 
      call_interactively(bind3(create_detector_setup, 
			       (string*)0, (string*)0, p_detector),
			 (string*)0, (string*)0);
  }
  DetectorMenu(const DetectorMenu& menu) { assign(menu); }
  DetectorMenu& operator=(const DetectorMenu& menu) { 
    if ( &menu == this ) return *this;
    assign(menu);
    return *this;
  }
private:
  void assign(const DetectorMenu& menu) {
    *((IstreamMenu*)this) = *((IstreamMenu*)&menu);
    hpt = menu.hpt;
    p_det = menu.p_det;
    // make relation
    initialize_sub_menus();
  }
  void initialize_sub_menus() {
    (*this)("sub-detector") = make_interactive_command
      (bind2(SetupMenuCaller(hpt), (istream**)0, p_det));
  }
};

#endif
