/* Contains several general tools for working with setup structure.

   1) Tools performing some action under all cells in the section, in the
   chamber or in the setup; under all sections in the chamber or in setup;
   under all chambers in setup.
   Usage:
   e.g. 
     for_each_cell(action, setup);
   or
     for_each_cell(action, chamber);
     for_each_cell(action, section);
   This function is overloaded for setup, chamber and section types.
   The "action" is any function object so that the expression
     action(icell); 
   is valid. Here icell is of type SomeSetup::ICell.

   Note it is not of the Cell type, since the action can need the Index.
   From the other hand, it is not IChamber_Cell for example, since it is
   supposed you use this method if your "action" can be performed only 
   inside the innermost loop (loop over all cells in this case).
   If it needs section information, then probably you should not use
   this for_each_... mechanism, as processing section information is
   better to be placed outside the loop over cells to save time. Thus
   your "action" should rely only on icell information. 

   Remark: let's consider for_each_section(action, setup);
   Note that the "inner" level (section) is given in the function name,
   the "outer" - via the function argument "setup".

   In detector_tools.hh the same functions are defined for iteration
   through the detector, e.g.
     for_each_cell(action, detector)
   etc.

----------------------------------------------------------------------
   In fact instead of 
     for_each_cell(action, setup);
   one may also write
     for_each_cell(action)(setup);
   for_each_cell with one argument (action) returns function object
   ForEachCell<Action> which can be applied to setup
     for_each_cell(action)(setup);
   The same function object can be applied also to chamber and section:
     for_each_cell(action)(chamber);
     for_each_cell(action)(section);
   
   The for_each_cell function with two arguments (action, x) returns
   void. It just calls
     for_each_cell(action)(x);
   i.e. it constructs ForEachCell<Action> object and immediately applies
   it to x.

     for_each_cell(action, x);
   can also be applied to the Detector, when x = detector, but
   there is NO appropriate for_each_cell(action)(detector); version.
   For generality one may use for_each_cell(action, x) in all cases,
   for x = section, chamber, setup, detector. In this sense this version
   is better.

......................................................................
   Why there is no for_each_cell(action)(detector); version.
   If one defined 
     ForEachCell<Action>::operator()(Detector& det)
   version then this would require that your "action" would be able to
   work with any Setup in the Detector. This would restrict the
   range of possible usages of ForEachCell<Action> class. 
......................................................................

   2) Setting the data in some cells to the default value.
      This can be useful e.g. in the beginning of each event.
      Usage, e.g.:
         reset_cell_data_in(setup,   default_value);
         reset_cell_data_in(chamber, default_value);
         reset_cell_data_in(section, default_value);

         reset_cell_data_in(detector,default_value); // if you include 
	                                 // detector_detector_tools.hh
      here "default_value" should be convertible to Cell::DataType.
      (your class CellType should have "data" field of type 
      CellType::DataType).
      Note that one function works in all 4 cases.

   3) Some geometry:
         setup_cell_center(isc); returns the center of the cell with
      ISection_Cell is equal to isc;
         setup_cell_rectangle(isc, &rectangle); calculates and puts the
      cell's rectangle at the given address &rectangle, again cell has 
      ISection_Cell isc.

   4) sort_setup_chambers_by_z(setup, &vector_vector_IChamber,
                               z_precision);
      This function sorts the chambers according to their z coordinates.
      It produces vector<vector<IChamber> > at the address
      &vector_vector_IChamber. This object stores groups of
      IChambers with the same z (inner vector) inside outer vector.
      The outer vector contains groups with z in ascending order.
      The "equality" of z coordinates is defined according to the 
      precision given by the parameter LessDouble ld (see LessDouble.hh).
      
    5) cout << identify_icc(icc);
      - identifies icc, i.e. prints out the chamber name, section # and 
      cell's index. Here icc is of any type IChamber_Cell.
      Quite useful tool (e.g. for debugging) to give information on 
      the cell.
   */

// Author V.Balagura (balagura@mail.desy.de)   (18.02.99).

#ifndef _setup_tools_hh
#define _setup_tools_hh 1

#include "ptset/setup.hh"
#include <functional>

/* ==================== for_each_... mechanism ====================*/

/* -------------------- ForEachCell -------------------- */
template <class Action>
struct ForEachCell {
  Action action;
  ForEachCell(const Action& x) : action(x) {}
  template <class CellT, class SectionT>
  void operator()(SetupSection<CellT, SectionT>& section) const { 
    for (typename SetupSection<CellT, SectionT>::ICell
	   icell=section.begin(); icell!=section.end(); ++icell) 
      action(icell); 
  }
  template <class CellT, class SectionT, class ChamberT>
  void operator()(SetupChamber<CellT, SectionT, ChamberT>& chamber) const
  { 
    for (typename SetupChamber<CellT, SectionT, ChamberT>::ISection
	   isection=chamber.begin(); isection!=chamber.end(); ++isection) 
      operator()(*isection);
  }
  template <class CellT, class SectionT, class ChamberT, class SetupT>
  void operator()(Setup<CellT, SectionT, ChamberT, SetupT>& setup) const
  { 
    for (typename Setup<CellT, SectionT, ChamberT, SetupT>::IChamber
	   ichamber=setup.begin(); ichamber!=setup.end(); ++ichamber) 
      operator()((*ichamber).second);
  }
};  
template <class Action>
inline ForEachCell<Action> for_each_cell(const Action& action) {
  return ForEachCell<Action>(action);
}
/* -------------------- ForEachSection -------------------- */
template <class Action>
struct ForEachSection {
  Action action;
  ForEachSection(const Action& x) : action(x) {}
  template <class CellT, class SectionT, class ChamberT>
  void operator()(SetupChamber<CellT, SectionT, ChamberT>& chamber) const
  { 
    for (typename SetupChamber<CellT, SectionT, ChamberT>::ISection
	   isection=chamber.begin(); isection!=chamber.end(); ++isection) 
      action(isection); 
  }
  template <class CellT, class SectionT, class ChamberT, class SetupT>
  void operator()(Setup<CellT, SectionT, ChamberT, SetupT>& setup) const
  { 
    for (typename Setup<CellT, SectionT, ChamberT, SetupT>::IChamber
	   ichamber=setup.begin(); ichamber!=setup.end(); ++ichamber) 
      operator()((*ichamber).second);
  }
};  
template <class Action>
inline ForEachSection<Action> for_each_section(const Action& action) {
  return ForEachSection<Action>(action);
}
/* -------------------- ForEachChamber -------------------- */
template <class Action>
struct ForEachChamber {
  Action action;
  ForEachChamber(const Action& x) : action(x) {}
  template <class CellT, class SectionT, class ChamberT, class SetupT>
  void operator()(Setup<CellT, SectionT, ChamberT, SetupT>& setup) const
  { 
    for (typename Setup<CellT, SectionT, ChamberT, SetupT>::IChamber
	   ichamber=setup.begin(); ichamber!=setup.end(); ++ichamber) 
      action(ichamber);
  }
};  
template <class Action>
inline ForEachChamber<Action> for_each_chamber(const Action& action) {
  return ForEachChamber<Action>(action);
}

/* ---------------- SetCellDataToDefaultValue ------------------ */
template <class AnyICell> 
struct SetCellDataToDefaultValue {
  typedef typename 
  iterator_traits<AnyICell>::value_type::second_type::DataType Data;
  Data data;
  SetCellDataToDefaultValue(const Data& default_data)
    : data(default_data) {}
  void operator()(AnyICell icell) const { (*icell).second.data = data; }
};

/* -------------------- reset_cell_data_in -------------------- */
template <class T, class InputDataType> 
inline void reset_cell_data_in(T& x, const InputDataType& data) {
  for_each_cell
    (SetCellDataToDefaultValue<typename T::ICell>(data), x);
  // InputDataType should be convertible to T::Cell::DataType
}

/* -------------------- Geometry -------------------- */
template <class ISC_OR_ICC>
inline Point setup_cell_center(const ISC_OR_ICC& isc) {
  return grid_center((*isc.icell).first, (*isc.isection).grid);
}
template <class ISC_OR_ICC>
inline void setup_cell_rectangle(const ISC_OR_ICC& isc, Rectangle* r) {
  Grid& grid = (*isc.isection).grid;
  r->a = (*isc.icell).first * grid;
  r->b = r->a + grid.step();
}
/* -------------------- sort_setup_chambers_by_z -------------------------- */
#include "ptset/LessDouble.hh"

template <class AnySetup, class IChamber>
inline void sort_setup_chambers_by_z
(AnySetup& setup, vector<vector<IChamber> >* chambers,
 double z_precision) {
  LessDouble ld(z_precision);
  typedef map<double, vector<typename AnySetup::IChamber>, LessDouble>
    Zmap;
  Zmap zmap(ld);
  for (typename AnySetup::IChamber ich=setup.begin(); ich!=setup.end(); ++ich)
    zmap[(*ich).second.z].push_back(ich);
  for ( Zmap::iterator iter=zmap.begin(); iter!=zmap.end(); ++iter )
    chambers->push_back((*iter).second);
}

/* -------------------- identify_icc -------------------- */
template <class ICC>
struct _ICC_Identification {
  ICC icc;
  _ICC_Identification(ICC fIcc) : icc(fIcc) {}

  friend ostream& operator<<(ostream& os, _ICC_Identification<ICC> id) {
    return os << "chamber \"" << (*id.icc.ichamber).first 
	      << "\", section #"
	      << id.icc.isection - (*id.icc.ichamber).second.begin() 
	      << ", cell " << (*id.icc.icell).first;
  }
};

template <class ICC>
inline _ICC_Identification<ICC> identify_icc(const ICC& icc) {
  return _ICC_Identification<ICC>(icc);
}

/* -------------------- for_each_...(action, x) -------------------- */
/* Cell */
template <class Action, class CellT, class SectionT>
inline void for_each_cell(const Action& action, 
			    SetupSection<CellT, SectionT>& section) {
  return ForEachCell<Action>(action)(section);
}
template <class Action, class CellT, class SectionT, class ChamberT>
inline void for_each_cell
(const Action& action, SetupChamber<CellT, SectionT, ChamberT>& chamber) {
  return ForEachCell<Action>(action)(chamber);
}
template <class Action, class CellT, class SectionT, class ChamberT, 
			      class SetupT>
inline void for_each_cell
(const Action& action, Setup<CellT, SectionT, ChamberT, SetupT>& setup) {
  return ForEachCell<Action>(action)(setup);
}

/* Section */
template <class Action, class CellT, class SectionT, class ChamberT>
inline void for_each_section
(const Action& action, SetupChamber<CellT, SectionT, ChamberT>& chamber) {
  return ForEachSection<Action>(action)(chamber);
}
template <class Action, class CellT, class SectionT, class ChamberT, 
			      class SetupT>
inline void for_each_section
(const Action& action, Setup<CellT, SectionT, ChamberT, SetupT>& setup) {
  return ForEachSection<Action>(action)(setup);
}

/* Chamber */
template <class Action, class CellT, class SectionT, class ChamberT, 
			      class SetupT>
inline void for_each_chamber
(const Action& action, Setup<CellT, SectionT, ChamberT, SetupT>& setup) {
  return ForEachChamber<Action>(action)(setup);
}

#endif
