#include "wax/OTrEventBuilder.hh"
#include "wax/OTrEventBuffer.hh"
#include "wax/OTrDetector.hh"
#include "wax/OTrDaq.hh"
#include "wax/OTrEvent.hh"
#include "wax/OTrRTRelation.hh"

#ifndef USE_ARTE 
 #include "log.h"
#endif

//__________________________________________________________________
// OTrEventBuilder
//
// Container for: 
// * OTrEventBuffer 
// * OTrEvent 
//
// Functionality:
// Building of the OTrEvent from the arte HITB bank (dst and monte
// carlo), from the DOTR bank or from the DAQ data files.
//
// Author: Wouter Hulsbergen && Ivar Siccama && Sascha Lanyov
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrEventBuilder) ;
#endif

bool OTrEventBuilder::fgPerformReconstruction = true ;

OTrEventBuilder::OTrEventBuilder()
  : fEventBuffer(0),fEvent(0),fDetector(0),fDataType(0),fNProcessedEvents(0) {}

OTrEventBuilder::OTrEventBuilder(OTrDetector* detector)
  : fEventBuffer(0),fEvent(0),fDetector(detector),fDataType(0),fNProcessedEvents(0)
{
#ifdef USE_ARTE
  if (ArteTable<EVHD>::begin()->exp <= 0) fDataType = kDF_HITB+kDS_MC ;
  else fDataType = kDF_DAQ+kDS_DOTR ;
#else
  fDataType = kDF_DAQ+kDS_FARM ;
#endif
  switch(GetDataFormat())
    {
    case kDF_DAQ:
      fEventBuffer = new OTrEventBuffer(GetDataStream()) ;
      break;
    case kDF_HITB:
      break;
    default:
      cout << "No such dataformat: " << GetDataFormat() ;
      break;
    }
}

OTrEventBuilder::~OTrEventBuilder()
{
 if(fEvent) delete fEvent ;
 if(fEventBuffer) delete fEventBuffer ;
}

void
OTrEventBuilder::EraseCurrentEvent()
{
  if(fEvent) delete fEvent;
  fEvent=0;
  switch(GetDataType())
    {
    case kDF_DAQ:
#ifdef USE_ARTE
      GetDetector()->ResetGEDEs();
#endif
      break;
    case kDF_HITB:
    default:
      break ;
    }
}

OTrEvent*
OTrEventBuilder::CreateNextEvent()
{ 
  int rc;
  EraseCurrentEvent() ;
  switch(GetDataFormat()) 
    {
    case kDF_DAQ:
      rc = GetEventBuffer()->NextFEDBuffers() ;
      if(rc==ETB_ok) {
	fEvent = DAQCreateEvent() ;
	fEvent->SortHits();
	fEvent->AppendHITB();
      }
      break ;
    case kDF_HITB:
      fEvent = HITBCreateEvent() ;
      fEvent->SortHits();// Note: not necessary when reading from HITB (IS)	
      // That's a pity, this is not true ... ??!! (WH)
      break;
    default:
      break ;
    }
  if(fEvent) {
#ifdef USE_ROOT
    fEvent->PerformHistogramming() ;
#endif
    if(GetPerformReconstruction()) {
      fEvent->SearchClusters() ;
      fEvent->SearchTrackCandidates() ;
      fEvent->AppendRSEG();
    }
  }
  return fEvent ;
}

OTrEvent*
OTrEventBuilder::DAQCreateEvent()
{
  OTrEvent* event = new OTrEvent(GetEventNumber()) ;
  GetDetector()->GetDaq()->IncreaseNProcessedEvents() ;//obsolete
  for (OTrEventBuffer::FedConstIterator fedIt(*GetEventBuffer());
       fedIt.valid(); fedIt.next())
    {
      OTrTdcBoard::ItemId_t tdcId = OTrTdcBoard::ItemId_t((*fedIt).GetId()) ;
      OTrTdcBoard* tdcIt = GetDetector()->GetDaq()->GetTdcBoard(tdcId) ;
      if(tdcIt) tdcIt->ProcessFedBuffer(*fedIt,event) ;
      // else cout << "OTrDaq::ProcessEvent(): could not find fedId " 
      // << tdcId << endl ;
    }
  fNProcessedEvents++ ;
  return event ;
}

#ifdef USE_ARTE
int OTrEventBuilder::GetRunNumber() {
  return ArteTable<EVHD>::begin()->run ;
}

int OTrEventBuilder::GetEventNumber() {
  return ArteTable<EVHD>::begin()->evt ;
}

OTrEvent*
OTrEventBuilder::HITBCreateEvent() 
{
  // Make a new OTrEvent out of the arte HITB bank. For Monte Carlo
  // purposes only.

  OTrEvent* event = new OTrEvent(GetEventNumber()) ;
  GetDetector()->GetDaq()->IncreaseNProcessedEvents() ;//obsolete
  for ( ArteTable<HITB>::iterator hitbIt = ArteTable<HITB>::begin(); 
	hitbIt != ArteTable<HITB>::end() ; hitbIt++ ) {
    //ArtePointer<HITB> hitb = ArtePointer<HITB>(hitbIt);
    //ArtePointer<GEDE> gede = ArtePointer<GEDE>(hitbIt->gede);
    if ( hitbIt->gede->cmp == GetOuterTrackerCMP() ) {  // select only OTr hits
      const OTrSection* secIt = GetDetector()->GetSectionFromGEDEIndex(hitbIt->gede.index()) ;
      if(secIt) {
	ArtePointer<DIGB> digb = 
	  ArtePointer<DIGB>(ArteRelationIterator<DIGB>(hitbIt->digb)) ;
	// Decoding of PART = 4*WIRE + MONOLAYER + 1
	int part = (digb->id)/10000;
	int digb_wirenr = (part-1)/4;  // wire (0..31)
	int monolayer = (part-1)%4;    // monolayer (0..3), or `tube'
	int otrwirenr = digb_wirenr*hitbIt->gede->nsl + monolayer;
	const OTrWire* wireIt = secIt->GetWire(otrwirenr);
	if(wireIt) {
	  OTrChannel* chanIt = wireIt->GetChannel() ;
	  if(chanIt) {
	    // 	    ArteRelationIterator<MIMP> imimp(hitb->mimp) ;
	    // 	    ArtePointer<MIMP> mimp(imimp) ;
	    // 	    float x0 = mimp->x - mimp->z*mimp->tx ;
	    // 	    float y0 = mimp->y - mimp->z*mimp->ty ;
	    // 	    bool targetConstraint = false ;
	    // 	    bool momentumConstraint = false ;//true ;
	    // 	    //cout << "momentum: " << mimp->p << endl ;
	    // 	    //cout << hitb << " " << hitb->z << " " << hitb->x << " " ;
	    // 	    if((!targetConstraint || (fabs(x0)<10 && fabs(y0)<10)) &&
	    // 	       (!momentumConstraint || (fabs(mimp->p)>10))) { 
	    float R = fabs(hitbIt->q) ;
	    float driftTime = chanIt->GetMCRTRelation()->GetT(R).GetValue() ;
	    OTrChannel::TdcCount_t tdccount = chanIt->GetTdcCount(driftTime) ;
	    //Channel->GetMCTdcCount(driftTime) ;
	    OTrHit* hit = chanIt->ProcessChannelBuffer( tdccount, event ) ;
	    if(hit) {
	      hit->SetHITB(hitbIt) ;
	      //cout << hit->GetWire()->GetRawZ() << " " << hit->GetWire()->GetRawX() << " " ;
	    } else {
	      cout << "Hit not accepted: " 
		   << R << " " << driftTime << " "
		   << int(tdccount) << " " << int(chanIt->GetMinimumTime()) << " "
		   << int(chanIt->GetMask()) << " "
		   << chanIt->GoodMask() << " " << endl ;
	    }
	  }
	} else {
	  cout << "******** Wire not found! gede,wirenr: " 
	       << hitbIt->gede.index() << " " << otrwirenr << endl ; 
	}
      } else {
	cout << "*** Section not found for GEDEIndex: " << hitbIt->gede.index() << endl;
      }
    }
  }
  fNProcessedEvents++ ;
  return event ;
}

#else // USE_ARTE

int 
OTrEventBuilder::GetRunNumber() {
  int runnr=0;
  if(GetEventBuffer()) runnr = GetEventBuffer()->GetLogHdr()->run ;
  return runnr ;
}

int OTrEventBuilder::GetEventNumber() {
  int evtnr=0;
  if(GetEventBuffer()) evtnr = GetEventBuffer()->GetLogHdr()->evt ;
  return evtnr ;
}

OTrEvent*
OTrEventBuilder::HITBCreateEvent() 
{
  // Dummy routine for running outside ARTE
  return 0;
}
#endif //USE_ARTE
