#include <math.h>
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include "wax/OTrParameterizedRTRelation.hh"
#include "wax/OTrRTResult.hh"

//__________________________________________________________________
// OTrParameterizedRTRelation
//
// Interface class for the parameterized rt-relations
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrParameterizedRTRelation) ;
#endif //USE_ROOT

//const OTrParameterizedRTRelation::ParameterizedRTRelationType OTrParameterizedRTRelation::fgType = OTrParameterizedRTRelation::kSpline ;

int OTrParameterizedRTRelation::fgCorrectionLevel=0;

OTrRTResult
OTrParameterizedRTRelation::CalcRTResult(const HepDouble R) const
{
  //cout << "ParameterizedRTRelation::CalcRTResult(const HepDouble R)" << endl ;
  OTrRTResult rtResult;
  const HepDouble dxdR = 2./(GetRMax()-GetRMin());
  HepDouble x = -1 + (R-GetRMin()) * dxdR ;
  if(x<-1) x = -1 ;
  else if(x>0.99999) x = 0.999999 ; //hahahah
 
  const int noTParameters = GetNoTParameters() ;
  vector<OTrFunctionValue> bfvT      = 
    CalcBasisFuncVector(x,noTParameters) ;
 
  HepVector bfvalue(noTParameters),bfderiv(noTParameters),
    bfderiv2(noTParameters);
  int i;
  for(i=0; i<noTParameters; i++)
    {
      bfvalue[i] = bfvT[i].GetValue() ;
      bfderiv[i] = bfvT[i].ddx * dxdR ;
      bfderiv2[i] = bfvT[i].d2dx2 * dxdR * dxdR ;
    }
  rtResult.fT.SetValue( dot(fTParameters,bfvalue) + GetT0() ) ;
  rtResult.fT.SetError( sqrt(fTCov.similarity(bfvalue)) ) ;
  rtResult.fdTdR.SetValue( dot(fTParameters,bfderiv) ) ;
  rtResult.fdTdR.SetError( sqrt(fTCov.similarity(bfderiv)) ) ;
  rtResult.fd2TdR2.SetValue( dot(fTParameters,bfderiv2) ) ;
  rtResult.fd2TdR2.SetError( sqrt(fTCov.similarity(bfderiv2)) ) ;

  if(R>GetRMax()) 
    rtResult.fT.SetValue( rtResult.fT.GetValue() + (R-GetRMax()) * 
    rtResult.fdTdR.GetValue() ) ;

  const int noSigma2Parameters = GetNoSigma2Parameters() ;
  vector<OTrFunctionValue> bfvSigma2  = 
    CalcBasisFuncVector(x,noSigma2Parameters) ;

  HepVector sbfvalue(noSigma2Parameters) ;
  for(i=0; i<noSigma2Parameters; i++)
    sbfvalue[i] = bfvSigma2[i].GetValue() ;

  rtResult.fSigma2.SetValue( dot(fSigma2Parameters,sbfvalue) ) ;
  rtResult.fSigma2.SetError( sqrt(fSigma2Cov.similarity(sbfvalue)) ) ;

  return rtResult ;
};

OTrFunctionValue
OTrParameterizedRTRelation::CalcFunctionValue(HepDouble var, HepDouble varMin,
					      HepDouble varMax,
					      const HepVector& 
					      varParameters) const
{
  //cout << "ParameterizedRTRelation::CalcValue(...)" << endl ;
  OTrFunctionValue V;
  const HepDouble dxdvar = 2./(varMax-varMin);
  const HepDouble x = -1 + (var-varMin) * dxdvar ;
  const int noParameters = varParameters.num_row() ;
  vector<OTrFunctionValue> bfv = CalcBasisFuncVector(x,noParameters) ;
  for(int i=0; i<noParameters; i++)
    {
      V.value += varParameters[i]* bfv[i].value ;
      V.ddx   += varParameters[i]* bfv[i].ddx * dxdvar ;
      V.d2dx2 += varParameters[i]* bfv[i].d2dx2 * dxdvar * dxdvar ;
    }
  return V;
}

HepVector
OTrParameterizedRTRelation::CalcParDeriv(HepDouble var, HepDouble varMin,
					 HepDouble varMax,
					 int size) const 
{
  //cout << "ParameterizedRTRelation::CalcParDeriv(size=" << size <<")" << endl;
  //const HepDouble dxdvar = 2/(varMax-varMin);
  const HepDouble x = -1 + 2*(var-varMin)/(varMax-varMin) ;
  vector<OTrFunctionValue> bfv = CalcBasisFuncVector(x,size) ;
  HepVector parDeriv(size) ;
  for(int i=0; i<size ; i++) parDeriv[i] = bfv[i].value ;
  return parDeriv;
}

HepVector
OTrParameterizedRTRelation::CalcdSigma2dPar(const HepDouble R) const {
  return CalcParDeriv(R,GetRMin(),GetRMax(),GetNoSigma2Parameters()) ;
};

HepVector
OTrParameterizedRTRelation::CalcdTdPar(const HepDouble R) const {
  return CalcParDeriv(R,GetRMin(),GetRMax(),GetNoTParameters()) ;
};

      
void
OTrParameterizedRTRelation::SetTParameters(const HepVector& TParameters,
					const HepSymMatrix& TCov) 
{
  //cout << "ParameterizedRTRelation::SetTParameters()" << endl ;
  fTParameters = TParameters ;
  if(TCov.num_row()==GetNoTParameters()) fTCov = TCov ;
};

void
OTrParameterizedRTRelation::SetSigma2Parameters(const HepVector& parameters, 
					     const HepSymMatrix& cov) 
{
  //cout << "ParameterizedRTRelation::SetSigma2Parameters" << endl ;
  fSigma2Parameters = parameters ;
  if(cov.num_row()==GetNoSigma2Parameters()) fSigma2Cov = cov;
}

void
OTrParameterizedRTRelation::UpdateTParameters(const HepVector& TParameters,
					   const HepSymMatrix& TCov)
{
  //cout << "ParameterizedRTRelation::UpdateTParameters()" << endl ;
  if(TParameters.num_row()!=0) SetTParameters(TParameters,TCov) ;
  RepairTParameters();
  //CorrectT0() ; // VERY IMPORTANT !!!! // ???????????
  FillTTable() ;
};

void
OTrParameterizedRTRelation::UpdateSigma2Parameters(const HepVector& parameters,
						const HepSymMatrix& cov)
{
  //cout << "ParameterizedRTRelation::UpdateSigma2Parameters()" << endl ;
  if(parameters.num_row()!=0) SetSigma2Parameters(parameters,cov) ;
  RepairSigma2Parameters();
  FillSigma2Table() ;
};

void
OTrParameterizedRTRelation::Write(ostream& ofs) const
{
  OTrRTRelation::Write(ofs) ;
  int row,col,num;
  num = GetNoTParameters() ;
  ofs  << setw(5) << setprecision(3) << num << endl ;
  for(row=0; row<num; row++)
    ofs << setw(15) << setprecision(10) << fTParameters[row] << endl;
  ofs << endl ;
  for(row=0; row<num; row++)
    for(col=0; col<=row; col++)
      ofs << setw(15) << setprecision(10) << fTCov[row][col] << endl;
  ofs << endl ;
  num = GetNoSigma2Parameters() ;
  ofs  << setw(5) << setprecision(3) << num << endl ;
  for(row=0; row<num; row++)
    ofs << setw(15) << setprecision(10) << fSigma2Parameters[row] << endl;
  ofs << endl ;
  for(row=0; row<num; row++)
    for(col=0; col<=row; col++)
      ofs << setw(15) << setprecision(10) << fSigma2Cov[row][col] << endl;
  return ;
};

void
OTrParameterizedRTRelation::Read(istream& ifs)
{
  //cout << "ParameterizedRTRelation::Read(istream& ifs)" << endl ;
  OTrRTRelation::Read(ifs) ;
  int row,col,num;
  ifs  >> num ;
  HepVector TParameters = HepVector(num) ;
  for(row=0; row<num; row++) ifs >> TParameters[row] ;
  HepSymMatrix TCov = HepSymMatrix(num) ;
  for(row=0; row<num; row++)
    for(col=0; col<=row; col++) ifs >> TCov[row][col] ;
  SetTParameters(TParameters,TCov) ;
  ifs  >> num ;
  HepVector Sigma2Parameters = HepVector(num) ;
  for(row=0; row<num; row++) ifs >> Sigma2Parameters[row] ;
  HepSymMatrix Sigma2Cov(num) ;
  for(row=0; row<num; row++)
    for(col=0; col<=row; col++) ifs >> Sigma2Cov[row][col] ;
  SetSigma2Parameters(Sigma2Parameters,Sigma2Cov) ;
  return ;
};

OTrParameterizedRTRelation::OTrParameterizedRTRelation() :
  OTrRTRelation(), tdTdParTable(GetNTableBins()), tdSigma2dParTable(GetNTableBins()) {
} ;

OTrParameterizedRTRelation::OTrParameterizedRTRelation(int noTparameters,
						       int noSigma2parameters, 
						       double rmin, double rmax) : 
  OTrRTRelation(rmin,rmax),fTParameters(noTparameters), fSigma2Parameters(noSigma2parameters), 
  fTCov(noTparameters,0), fSigma2Cov(noSigma2parameters,0),
  tdTdParTable(GetNTableBins()), tdSigma2dParTable(GetNTableBins()) {
    //FillTables() ;
} ;

OTrParameterizedRTRelation::~OTrParameterizedRTRelation() {} ;

void
OTrParameterizedRTRelation::FillBasisFuncTables() {
  //cout << "ParameterizedRTRelation::FillTables()" << endl ;
  int xbin;
  HepDouble R;
  for(xbin=0; xbin < GetNTableBins(); xbin++)
    {
      R = GetR(xbin) ;
      tdTdParTable[xbin] = CalcParDeriv(R,GetRMin(),GetRMax(),
					GetNoTParameters()) ;
      tdSigma2dParTable[xbin] = CalcParDeriv(R,GetRMin(),GetRMax(),
					     GetNoSigma2Parameters()) ;
    };
};

#ifdef USE_ROOT
#include "TGraphErrors.h"

TGraphErrors*
OTrParameterizedRTRelation::GetTGraph(int nPoints) {
  TGraphErrors* graph=new TGraphErrors(nPoints) ;
  if(nPoints<2) nPoints=100 ;
  float deltar=(GetRMax()-GetRMin())/(nPoints-1) ;
  float r ; OTrRTResult rt ;
  for(int i=0; i<nPoints; i++) {
    r=GetRMin() + deltar * i ;
    rt = CalcRTResult(r) ;
    graph->SetPoint(i,r,rt.GetT().GetValue()) ;
    graph->SetPointError(i,0,rt.GetT().GetError()) ;
    //graph->SetPoint(i,r,GetT(r).GetValue()) ;
  };
  return graph ;
};

TGraphErrors*
OTrParameterizedRTRelation::GetSigmaGraph(int nPoints) {
  TGraphErrors* graph=new TGraphErrors(nPoints) ;
  if(nPoints<2) nPoints=100 ;
  float deltar=(GetRMax()-GetRMin())/(nPoints-1) ;
  float r ; OTrRTResult rt ;
  for(int i=0; i<nPoints; i++) {
    r=GetRMin() + deltar * i ;
    rt = CalcRTResult(r) ;
    graph->SetPoint(i,r,rt.GetSigma().GetValue()) ;
    graph->SetPointError(i,0,rt.GetSigma().GetError()) ;
  };
  return graph ;
};
#endif
