#include <CLHEP/Utilities/fortran.h>
extern "C" float FORTRAN_ROUTINE( prob )( const float& x,
                                          const int& n );
#ifdef USE_ROOT
#include <TSystem.h>
#include <TCanvas.h>
#include <TH1.h>
#include <TH2.h>
#include <TObjArray.h>
#include <TMinuit.h>
#endif

#include "wax/OTrTZeroCalib.hh"
#include "wax/OTrDaq.hh"
#include "wax/OTrSection.hh"
#include "wax/OTrDetector.hh"
#include "wax/OTrChannelHistogrammer.hh"
#include "wax/OTrTdcHistogram.hh"

//__________________________________________________________________
// OTrTZeroCalib
//
// Functionality:
// All stuff having to do with t0 calibration ...
//
// Author: Ivar Siccama & Wouter Hulsbergen
//__________________________________________________________________

#ifdef __OTR_CLASSDEF__
ClassImp(OTrTZeroCalib) ;
#endif 

OTrTZeroCalib::OTrTZeroCalib() :
  fDaq(0), fDetector(0), fSuperLayerId(2), fRef5mmH1(0), fRef10mmH1(0) 
{
}

OTrTZeroCalib::OTrTZeroCalib( OTrDaq* daq, OTrDetector* det ) 
  : fDaq(daq), fDetector(det), fSuperLayerId(2), fRef5mmH1(0), fRef10mmH1(0)  
{
}

OTrTZeroCalib::~OTrTZeroCalib() {} ;

#ifdef USE_ROOT
vector<double>* gReferenceHistogram;

double timeHistoFunction(double* x, double* par)
{
  if(gReferenceHistogram==0) {
    cout << "No reference spectrum" << endl;
    return 0;
  };

  double& T = x[0] ;
  double& T0 = par[0];
  double& norm = par[1] ;
  //double& background = par[2] ;

  float timebin = 0.5 + OTrChannel::GetNTimeBins()*
    (T+T0-OTrChannel::GetMinimumTimeAxis())/
    (OTrChannel::GetMaximumTimeAxis()-
     OTrChannel::GetMinimumTimeAxis()) ;

  int timebin1 = (int(timebin))%OTrChannel::GetNTimeBins() ;
  int timebin2 = (int(timebin+1))%OTrChannel::GetNTimeBins() ;
  float v1 = (*gReferenceHistogram)[timebin1] ;
  float v2 = (*gReferenceHistogram)[timebin2] ;

  return norm*(v1 + (v2-v1) *
	       (timebin-timebin1) / float(timebin2-timebin1)) ;
};

int OTrTZeroCalib::Calibration( const int method, bool inter  ) 
{
  if ( method == 0 || method > 6 ) {
    cout << "Usage:  Calibration( const int method )" << endl;
    cout << " method = 1 : fit to a reference spectrum" << endl;
    cout << " method = 2 : fit to two erfc functions " << endl; 
    cout << " method = 3 : fit to an asymmetric Gaussian" << endl;
    cout << " method = 4 : average TDC value" << endl;
    cout << " method = 5 : bin with maximum content (testpulse runs)" << endl;
    cout << " method = 6 : as (5) but with Gaussian fit around max" << endl;
    return 0;
  }

  const double cAbsT0Max = 50 ;
  const double cAbsT0MaxAxis = cAbsT0Max+5 ;
  TCanvas* fitCanvas = new TCanvas("mycanvas","") ;
  TCanvas* fitCanvas2 = new TCanvas("mycanvas2","") ;

  fitCanvas->Divide(3,2) ;
  TObjArray* allHistos = new TObjArray() ;

  TH1F* deltaTZero5mmH1 =
    new TH1F("deltaTZero5mm","",100,-cAbsT0MaxAxis,cAbsT0MaxAxis) ;
  deltaTZero5mmH1->SetDirectory(0) ;
  allHistos->Add(deltaTZero5mmH1) ;

  TH1F* curTZero5mmH1 =
    new TH1F("curTZero5mm","",100,-cAbsT0MaxAxis,cAbsT0MaxAxis) ;
  curTZero5mmH1->SetDirectory(0) ;
  allHistos->Add(curTZero5mmH1) ;

  TH1F* deltaTZero10mmH1 = 
    new TH1F("deltaTZero10mm","",100,-cAbsT0MaxAxis,cAbsT0MaxAxis) ;
  deltaTZero10mmH1->SetDirectory(0) ;
  allHistos->Add(deltaTZero10mmH1) ;

  TH1F* curTZero10mmH1 = 
    new TH1F("curTZero10mm","",100,-cAbsT0MaxAxis,cAbsT0MaxAxis) ;
  curTZero10mmH1->SetDirectory(0) ;
  allHistos->Add(curTZero10mmH1) ;

  TH1F *curTZeroH1, *deltaTZeroH1 ;

  TH2F *probvstzero5mm = new TH2F("probvstzero5mm","",100,0,1,50,-50,50);
  TH2F *probvstzero10mm = new TH2F("probvstzero10mm","",100,0,1,50,-50,50);
  TH2F *errvstzero5mm = new TH2F("errvstzero5mm","",100,0,100,50,-50,50);
  TH2F *errvstzero10mm = new TH2F("errvstzero10mm","",100,0,100,50,-50,50);
  TH2F *probvstzero;
  TH2F *errvstzero;

  bool first=true;
  bool converged=true ;

  TF1* f1 = 0;
  int nFitPar;
  int nmaxiter = 1;
  const int minNHits=100;
  double rangemin5mm,  rangemax5mm;
  double rangemin10mm, rangemax10mm;
  double rangemin, rangemax;
  if ( method == 1 ) {
    nFitPar = 2;
    f1 = new TF1("f1", timeHistoFunction, 0, 256, nFitPar) ;
    nmaxiter = 5;
    rangemin5mm  = 100; rangemax5mm  = 240;
    rangemin10mm = 50;  rangemax10mm = 220;
  } 

  double startpar5mm[nFitPar];
  double startpar10mm[nFitPar];
  double* startpar;

  int nconverged5mm, nconverged10mm, nfiterr5mm, nfiterr10mm;
  int nSteps = 0;
  char user_input = ' ';

  do {
    if ( user_input == 'q' ) break;
    //// ********** FIT ITERATION ********** ////
    if ( method == 1 ) {
      FillReferenceSpectra( nSteps );
      char* opt="";
      if(!first) opt="same" ;
      fitCanvas->cd(1); fRef5mmH1->Draw(opt) ; 
      fitCanvas->cd(4); fRef10mmH1->Draw(opt) ;
      fitCanvas->Update() ;
    }
    deltaTZero5mmH1->Reset() ;
    curTZero5mmH1->Reset() ;
    deltaTZero10mmH1->Reset() ;
    curTZero10mmH1->Reset() ;
    probvstzero5mm->Reset();
    probvstzero10mm->Reset();
    errvstzero5mm->Reset(); 
    errvstzero10mm->Reset();
 
    double sum5mmT0=0; int n5mm=0;
    double sum10mmT0=0; int n10mm=0;
    double* sumT0; int *n;
    OTrTdcHistogram* TDChisto;
    double sum5mmDelta2=0;
    double sum10mmDelta2=0;
    double* sumDelta2 ;
    nconverged5mm = 0;
    nconverged10mm = 0;
    nfiterr5mm = 0;
    nfiterr10mm = 0;
    int* nconverged;
    int* nfiterr;

    for ( vector<OTrSuperLayer>::iterator sulIt =  
	    fDetector->GetSuperLayers().begin();
	  sulIt != fDetector->GetSuperLayers().end(); sulIt++ ) { 
      //if ( sulIt->GetSuperLayerId() != GetSuperLayerId() ) continue;
      if ( user_input == 'q' ) break;
      for ( OTrSuperLayer::LayerIterator layIt(*sulIt) ; layIt.valid(); 
	    layIt.next() ) { 
	if ( user_input == 'q' ) break;
	for ( OTrLayer::SectionIterator secIt(*layIt) ; secIt.valid();
	      secIt.next() ) {
	  if ( user_input == 'q' || user_input == 'n' ) break;
	  if ( (*secIt).GetEPitch()==OTrGede::kFiveMM) {
	    gReferenceHistogram= &fReferenceHistogram5mm ;
	    deltaTZeroH1 = deltaTZero5mmH1 ;
	    curTZeroH1 = curTZero5mmH1 ;
	    n =&n5mm; sumT0=&sum5mmT0; sumDelta2=&sum5mmDelta2 ;
	    if ( f1 ) {
	      probvstzero = probvstzero5mm ;
	      errvstzero = errvstzero5mm ;
	      rangemin = rangemin5mm; 
	      rangemax = rangemax5mm;
	      nconverged = &nconverged5mm;
	      nfiterr = &nfiterr5mm;
	      startpar = startpar5mm;
	    }
	  } else {
	    gReferenceHistogram= &fReferenceHistogram10mm ;
	    deltaTZeroH1 = deltaTZero10mmH1 ;
	    curTZeroH1 = curTZero10mmH1 ;
	    n =&n10mm; sumT0=&sum10mmT0; sumDelta2=&sum10mmDelta2 ;
	    if ( f1 ) {
	      probvstzero = probvstzero10mm ;
	      errvstzero = errvstzero10mm ;
	      rangemin = rangemin10mm; 
	      rangemax = rangemax10mm;
	      nconverged = &nconverged10mm;
	      nfiterr = &nfiterr10mm;
	      startpar = startpar10mm;
	    }
	  }	
	  
	  for ( vector<OTrWire>::const_iterator wireIt = 
		  (*secIt).GetWires().begin();
		wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	    OTrChannel* chanIt = wireIt->GetChannel();
	    if(!chanIt) continue ;
	    if ( !(*chanIt).GoodMask() ) continue;
	    if((*chanIt).GetNHit() < minNHits ) continue;
	    float oldT0=(*chanIt).GetTZero().GetValue() ;
	    float T0 = 0;
	    float T0error = 0;
	    if ( method == 4 ) { 
	      T0 = -(*chanIt).GetAverageFromTimeH1() ;
	      //	      cout << "T0 = " << T0 << endl;
	      //getchar();
	    }
	    TH1D* tdch1 = chanIt->GetTimeH1() ;
	    TDChisto = new OTrTdcHistogram( *tdch1 ) ;
	    delete tdch1 ;

	    if ( method == 2 || method == 3 || 
		 method == 5 || method == 6 ) {
	      OTrFObservable newT0;
	      if ( method == 2 ) {
		newT0 = TDChisto->FitTwoErrorFunctions();
	      } else if ( method == 3 ) {
		OTrFObservable width;
		newT0 = TDChisto->FitAsymmetricGaussian( width );
	      } else if ( method == 5 || method == 6 ) {
		bool useFit = false;
		if ( method == 6 ) useFit = true; 
		newT0 = TDChisto->GetPeakValue( useFit );
	      }
	      cout << "new T0, T0error = " << newT0.GetValue() 
		   << " " << newT0.GetError() << endl;
	      if ( inter ) {
		fitCanvas2->cd();
		TDChisto->Draw();
		fitCanvas2->Update();
		cout << "press any key" << endl;
		user_input = getchar();
		if ( user_input == 'q' || user_input == 'n' ) break;
	      }	
	    }
	    
	    if ( method == 1 ) {
	      f1->SetRange( rangemin, rangemax ) ;
	      if ( method == 1 ) {
		f1->SetParameter(0,0);
		f1->SetParameter(1,1E-3) ;
		f1->SetParLimits(0,-cAbsT0Max,cAbsT0Max) ;
	      } 
	      if ( gMinuit ) gMinuit->SetMaxIterations( 2000 );
	      
	      //if ( nSteps == 4 ) {
	      if ( inter ) {
		TDChisto->Fit("f1","RB") ;
		fitCanvas2->cd();
		TDChisto->Draw();
		fitCanvas2->Update();
		user_input = getchar();
		if ( user_input == 'q' || user_input == 'n' ) break;
	      } else {
		TDChisto->Fit("f1","QR0B") ;
	      }

	      Ssiz_t indx = gMinuit->fCstatu.Index("CONVERGED");
	      //cout << "index = " << indx << endl;
	      if ( indx >= 0 ) {
		(*nconverged)++;
	      } else {
		(*nfiterr)++;
	      }
	      
	      delete TDChisto ;
	      
	      if ( method == 1 ) {
		T0 = f1->GetParameter(0) ;
		T0error = f1->GetParError(0);
	      } 
	      float chi2 = f1->GetChisquare() ;
	      //	    float prb = f1->GetProb() ;
	      int ndof   = f1->GetNumberFitPoints() - nFitPar ;
	      if(ndof>0) {
		float prb = FORTRAN_ROUTINE( prob )( chi2, ndof ) ;
		probvstzero->Fill( prb, T0) ;
	      }
	      errvstzero->Fill( T0error, T0 ) ;
	      //cout << " prob, chi2, t0 = " << prb << " " 
	      // << chi2 << " " << T0 << " +/- "  
	      // << err << " " 
	      // << f1->GetParameter(1) << flush;
	    }; 
	    float deltaT0 = T0-oldT0 ;
	    deltaTZeroH1->Fill(deltaT0) ;
	    curTZeroH1->Fill(T0) ;
	    // set T0 values with a suspiciously large error to 49
	    //if ( err > 10 ) T0 = 49;
	    (*chanIt).SetTZero( OTrFObservable(T0,T0error) ) ;
	    if ( T0 != 0 ) {
	      (*n)++; 
	      (*sumT0)+=T0; 
	      *sumDelta2 += deltaT0*deltaT0;
	    }
	    gSystem->ProcessEvents();
	  }
	  cout << (*secIt).GetSuperLayerId() << "." 
	       << (*secIt).GetLayerId() << "."
	       << (*secIt).GetSectionId() << "|" << flush ;
	}
      }
    }
    cout << endl ;
    fitCanvas->cd(2); deltaTZero5mmH1->Draw() ;
    fitCanvas->cd(5); deltaTZero10mmH1->Draw() ;
    fitCanvas->cd(3); curTZero5mmH1->Draw() ;
    fitCanvas->cd(6); curTZero10mmH1->Draw() ;
    fitCanvas->Update() ;

    double av5mmT0 = sum5mmT0/n5mm;
    double av10mmT0 = sum10mmT0/n10mm;
    cout << "nsteps = " << nSteps << endl;
    cout << "nconverged = " << nconverged5mm << " " << nconverged10mm << endl; 
    cout << "nfiterr    = " << nfiterr5mm << " " << nfiterr10mm << endl;
    cout << "Average T0: " << av5mmT0 << " " << av10mmT0 << endl;
    //SubtractAverage( av5mmT0, av10mmT0 );
    SubtractAverageMC02() ;

    first=false ;
    double av5mmDelta2 = sum5mmDelta2/n5mm -av5mmT0*av5mmT0 ;
    double av10mmDelta2 = sum10mmDelta2/n10mm-av10mmT0*av10mmT0 ;
    cout << "Av Delta2 : " << av5mmDelta2 << " " << av10mmDelta2 << endl ;
    converged = av5mmDelta2<0.1 && av10mmDelta2<0.1;
    //plot(10,6,false) ;
    nSteps++;
    if ( nSteps >= nmaxiter ) break;
  } while( !converged || nSteps == 1) ;

  if ( method == 1 ) {
    fDaq->WriteTable("caldata/tzero/TZeroFitRefNew.dat",OTrDaq::kTZeroTable) ;
  } else if ( method == 2 ) {
    fDaq->WriteTable("caldata/tzero/TZeroFitErfNew.dat",OTrDaq::kTZeroTable) ;
  } else if ( method == 3 ) {
    fDaq->WriteTable("caldata/tzero/TZeroFitGauNew.dat",OTrDaq::kTZeroTable) ;    
  } else if ( method == 4 ) {
    fDaq->WriteTable("caldata/tzero/TZeroAvNew.dat",OTrDaq::kTZeroTable) ; 
  } else if ( method == 5 ) {
    fDaq->WriteTable("caldata/tzero/TZeroMaxNew.dat",OTrDaq::kTZeroTable) ; 
  } else if ( method == 6 ) {
    fDaq->WriteTable("caldata/tzero/TZeroMaxGNew.dat",OTrDaq::kTZeroTable) ; 
  }

  /*
  TObjArrayIter it(allHistos) ;
  TObject* histo ;
  while((histo=it.Next()))  {
    allHistos->Remove(histo) ;
    histo->Delete() ;
  }
  fitCanvas->Delete() ; 
  */
  return 0;
}

void OTrTZeroCalib::FillReferenceSpectra( const int nSteps )
{
  // FILL REFERENCE SPECTRA 
  fReferenceHistogram5mm  = vector<double>(OTrChannel::GetNTimeBins()) ; 
  fReferenceHistogram10mm = vector<double>(OTrChannel::GetNTimeBins()) ;
  for( int i = 0; i < OTrChannel::GetNTimeBins(); i++ ) {
    fReferenceHistogram5mm[i]=0;
    fReferenceHistogram10mm[i]=0;
  };
    
  vector<double>* fRef;    
  for ( vector<OTrSuperLayer>::iterator sulIt =  
	  fDetector->GetSuperLayers().begin();
	sulIt != fDetector->GetSuperLayers().end(); sulIt++ ) {
    // So we only take MC02 !! good ...
    if ( sulIt->GetSuperLayerId() != GetSuperLayerId() ) continue;
    for ( OTrSuperLayer::LayerIterator layIt(*sulIt) ; layIt.valid(); 
	  layIt.next() ) {
      for ( OTrLayer::SectionIterator secIt(*layIt) ; secIt.valid();
	    secIt.next() ) {
	if ( (*secIt).GetEPitch()==OTrGede::kFiveMM) 
	  fRef = &fReferenceHistogram5mm ;
	else
	  fRef = &fReferenceHistogram10mm ;
	for ( vector<OTrWire>::const_iterator 
		wireIt = (*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  if(!chanIt) continue ;
	  if ( !(*chanIt).GoodMask() ) continue;
	  if ( (*chanIt).GetNHit() <= 0) continue;
	  int trbin = -(*chanIt).GetTZeroBin() ;
	  if(trbin<0) trbin+=OTrChannel::GetNTimeBins() ;
	  for(int bin=0; bin<OTrChannel::GetNTimeBins(); bin++,trbin++) {
	    (*fRef)[bin]+=
	      (*chanIt).GetTimeHistogram()[trbin%OTrChannel::GetNTimeBins()] ;
	  }
	  // Let's not do it cyclic !
	  //int t0bin = (*chanIt).GetTZeroBin() ;
	  //int minbin = max(t0bin,0) ;
	  //int maxbin = min(OTrChannel::GetNTimeBins(),
	  //  	       OTrChannel::GetNTimeBins()+t0bin) ;
	  //for(int bin=minbin; bin<maxbin; bin++) {
	  //  (*fRef)[bin]+=
	  //    (*chanIt).GetTimeHistogram()[bin-t0bin] ;
	  //};
	  //if(first&&t0bin!=0) cout << "SHIT!!!!!!! " << t0bin << endl ;
	};
	gSystem->ProcessEvents();
      }
    }
  }

  //if ( fRef5mmH1 ) { delete fRef5mmH1; }
  //if ( fRef10mmH1 ) { delete fRef10mmH1; }
  char name[50];
  sprintf( name, "%s%d", "ref5mm_", nSteps );
  fRef5mmH1 = CreateReferenceHistogram(fReferenceHistogram5mm,name,
				       nSteps+2);
  sprintf( name, "%s%d", "ref10mm_", nSteps );
  fRef10mmH1 = CreateReferenceHistogram(fReferenceHistogram10mm,name,
				       nSteps+2);
}    

TH1F* OTrTZeroCalib::CreateReferenceHistogram( vector<double>& ref, 
					       char* name, int color ) 
{
  TH1F* hist = new TH1F(name,"",OTrChannel::GetNTimeBins(),
			OTrChannel::GetMinimumTimeAxis(),
			OTrChannel::GetMaximumTimeAxis()) ;
  for(int bin=0; bin<OTrChannel::GetNTimeBins(); bin++) 
    hist->SetBinContent(bin+1,ref[bin]);
  hist->SetLineColor(color);  
  return hist;
}

void OTrTZeroCalib::CalibFromResiduals()
{
  vector<OTrChannel*> adjustedChannels05mm ;
  double sumdeltaTZero05mm(0) ;
  double sumdeltaTZero205mm(0) ;
  vector<OTrChannel*> adjustedChannels10mm ;
  double sumdeltaTZero10mm(0) ;
  double sumdeltaTZero210mm(0) ;

  vector<OTrChannel*>* adjustedChannels ;
  double* sumdeltaTZero ;
  double* sumdeltaTZero2 ;

  for ( OTrDetector::SuperLayerContainer::iterator sulIt =  
	  fDetector->GetSuperLayers().begin();
	sulIt != fDetector->GetSuperLayers().end(); sulIt++ )
    for ( OTrSuperLayer::LayerContainer::iterator
	    layIt = (*sulIt).GetLayers().begin() ;
	  layIt != (*sulIt).GetLayers().end() ; layIt++)
      for ( OTrLayer::SectionContainer::iterator
	      secIt = (*layIt).GetSections().begin() ;
	    secIt != (*layIt).GetSections().end() ; secIt++) {
	if ( (*secIt).GetEPitch()==OTrGede::kFiveMM) {
	  adjustedChannels = &adjustedChannels05mm ;
	  sumdeltaTZero    = &sumdeltaTZero05mm ;
	  sumdeltaTZero2   = &sumdeltaTZero205mm ;
	} else {
	  adjustedChannels = &adjustedChannels10mm ;
	  sumdeltaTZero    = &sumdeltaTZero10mm ;
	  sumdeltaTZero2   = &sumdeltaTZero210mm ;
	}
	for ( OTrSection::WireContainer::iterator wireIt = 
		(*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  if(chanIt) {
	    double deltaTZero = chanIt->GetHistogrammer()->UpdateTZero() ;
	    if(deltaTZero>-500) {
	      adjustedChannels->push_back(chanIt) ;
	      *sumdeltaTZero  +=deltaTZero;
	      *sumdeltaTZero2 +=deltaTZero*deltaTZero;
	    }
	  }
	}
      }
  if(adjustedChannels05mm.size()) {
    double avDeltaTZero = sumdeltaTZero05mm/adjustedChannels05mm.size() ;
    for(vector<OTrChannel*>::iterator chanIt=adjustedChannels05mm.begin() ;
	chanIt!=adjustedChannels05mm.end() ; chanIt++) {
      OTrFObservable tzero = (*chanIt)->GetTZero() ;
      tzero.SetValue(tzero.GetValue()-avDeltaTZero) ;
      (*chanIt)->SetTZero(tzero);
    }
    cout << "Number of adjusted 05mm tzeros, average, width : " 
	 << adjustedChannels05mm.size() << " " 
	 << avDeltaTZero << " " 
	 << sqrt(sumdeltaTZero205mm/adjustedChannels05mm.size()-sqr(avDeltaTZero)) 
	 << endl ;
  }
  if(adjustedChannels10mm.size()) {
    double avDeltaTZero = sumdeltaTZero10mm/adjustedChannels10mm.size() ;
    for(vector<OTrChannel*>::iterator chanIt=adjustedChannels10mm.begin() ;
	chanIt!=adjustedChannels10mm.end() ; chanIt++) {
      OTrFObservable tzero = (*chanIt)->GetTZero() ;
      tzero.SetValue(tzero.GetValue()-avDeltaTZero) ;
      (*chanIt)->SetTZero(tzero);
    }
    cout << "Number of adjusted 10mm tzeros, average, width : " 
	 << adjustedChannels10mm.size() << " " 
	 << avDeltaTZero << " " 
	 << sqrt(sumdeltaTZero210mm/adjustedChannels10mm.size()-sqr(avDeltaTZero)) 
	 << endl ;
  }
  SubtractAverageMC02() ;
}

void OTrTZeroCalib::SubtractAverageMC02()
{
  // Hi Ivar ! I modified the SubtractAverage routine: now only the
  // average T0 value of the MC02 chamber is kept zero. The reason is
  // that otherwise the T0s change substantially every time we add a
  // new superlayer.
  
  OTrSuperLayer* mc2sl = fDetector->FindSuperLayer(2) ;
  if(mc2sl) {
    double sumTZero05mm(0) ;
    double sumTZero10mm(0) ;
    double* sumTZero ;
    int nTZero05mm(0) ;
    int nTZero10mm(0) ;
    int* nTZero ;
    for ( OTrSuperLayer::LayerIterator layIt(*mc2sl) ; layIt.valid(); 
	  layIt.next() )
      for ( OTrLayer::SectionIterator secIt(*layIt) ; secIt.valid();
	    secIt.next() ) {
	if ( (*secIt).GetEPitch()==OTrGede::kFiveMM) {
	  sumTZero = &sumTZero05mm ;
	  nTZero   = &nTZero05mm ;
	} else {
	  sumTZero = &sumTZero10mm ;
	  nTZero   = &nTZero10mm ;
	} 
	for ( vector<OTrWire>::const_iterator wireIt = 
		(*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  //if(chanIt && chanIt->GetNHit() > 0) {
	  *sumTZero += chanIt->GetTZero().GetValue() ;
	  (*nTZero)++ ;
	  //}
	}
      }
    double avTZero05mm = sumTZero05mm/nTZero05mm ;
    double avTZero10mm = sumTZero10mm/nTZero10mm ;
    cout << "Average t0 in MC02 was : " << avTZero05mm << " " << avTZero10mm << endl ;
    SubtractAverage(avTZero05mm,avTZero10mm) ; 
  } else {
    cout << "No superlayer MC2: cannot substract average" << endl ;
  }
} 

void OTrTZeroCalib::SubtractAverage(const double av5mm, 
				    const double av10mm )
{  
  // Substract given arguments from tzero values ...

  double avT0;

  for ( OTrDetector::SuperLayerContainer::iterator sulIt =  
	  fDetector->GetSuperLayers().begin();
	sulIt != fDetector->GetSuperLayers().end(); sulIt++ )
    //if ( sulIt->GetSuperLayerId() != GetSuperLayerId() ) continue;
    for ( OTrSuperLayer::LayerContainer::iterator
	    layIt = (*sulIt).GetLayers().begin() ;
	  layIt != (*sulIt).GetLayers().end() ; layIt++)
      for ( OTrLayer::SectionContainer::iterator
	      secIt = (*layIt).GetSections().begin() ;
	    secIt != (*layIt).GetSections().end() ; secIt++) {
	
	if ( (*secIt).GetEPitch()==OTrGede::kFiveMM) avT0 = av5mm;
	else avT0 = av10mm;
	
	for ( OTrSection::WireContainer::iterator wireIt = 
		(*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  //if (!chanIt || (*chanIt).GetNHit()<= 0) continue; 
	  if(chanIt) {
	    OTrFObservable oldT0 =  (*chanIt).GetTZero() ;
	    double T0 = oldT0.GetValue() - avT0 ;
	    //if ( oldT0.GetValue() == 0 ) T0 = 0; 
	    (*chanIt).SetTZero(OTrFObservable( T0, oldT0.GetError() ));
	  }
	}
      }
}

void OTrTZeroCalib::Monitor()
{  
  // Create some monitoring histograms ...

  static TCanvas* c1 = new TCanvas("tzero_distribution","tzero_distribution") ;
  //c1->Draw();

  static TH1F* tzero5mmH1 = new TH1F("tzero5mm","",100,-25,25);
  tzero5mmH1->Reset() ;
  static TH1F* tzero10mmH1 = new TH1F("tzero10mm","",100,-25,25);
  tzero10mmH1->Reset() ;
  TH1F* tzeroH1;

  int nbins = OTrChannel::GetNTimeBins();
  float tmin = OTrChannel::GetMinimumTimeAxis();
  float tmax = OTrChannel::GetMaximumTimeAxis();
  // total TDC histogram before T0 correction
  static TH1F* before5mmH1 = new TH1F("before5mm","5 mm",nbins,tmin,tmax);
  before5mmH1->Reset() ;
  static TH1F* before10mmH1 = new TH1F("before10mm","10 mm",nbins,tmin,tmax);
  before10mmH1->Reset() ;
  TH1F* beforeH1;   

  // after T0 correction
  static TH1F* after5mmH1 = new TH1F("after5mm","5 mm",nbins,tmin,tmax);
  after5mmH1->Reset() ;
  static TH1F* after10mmH1 = new TH1F("after10mm","10 mm",nbins,tmin,tmax);
  after10mmH1->Reset() ;
  TH1F* afterH1;

  for ( OTrDetector::SuperLayerContainer::const_iterator sulIt =  
	  fDetector->GetSuperLayers().begin();
	sulIt != fDetector->GetSuperLayers().end(); sulIt++ ) {
    if ( sulIt->GetSuperLayerId() != GetSuperLayerId() ) continue;
    for ( OTrSuperLayer::LayerContainer::const_iterator
	    layIt = (*sulIt).GetLayers().begin() ;
	  layIt != (*sulIt).GetLayers().end() ; layIt++)
      for ( OTrLayer::SectionContainer::const_iterator
	      secIt = (*layIt).GetSections().begin() ;
	    secIt != (*layIt).GetSections().end() ; secIt++) {
	if ( (*secIt).GetEPitch()==OTrGede::kFiveMM) {
	  tzeroH1 = tzero5mmH1;
	  afterH1 = after5mmH1;
	  beforeH1 = before5mmH1;
	} else {
	  tzeroH1 = tzero10mmH1;
	  afterH1 = after10mmH1;
	  beforeH1 = before10mmH1;
	}
	for ( OTrSection::WireContainer::const_iterator 
		wireIt = (*secIt).GetWires().begin();
	      wireIt != (*secIt).GetWires().end(); wireIt++ ) {
	  OTrChannel* chanIt = wireIt->GetChannel();
	  if (!chanIt || (*chanIt).GetNHit()<= 0) continue; 
	  float T0 = (*chanIt).GetTZero().GetValue() ;
	  tzeroH1->Fill( T0 );
	  if ( (*chanIt).GoodMask() && (*chanIt).GetOccupancy() < 0.15 
	       && T0 < 25 ) {
	    TH1D* h1 = (*chanIt).GetTimeH1_B( T0 );
            afterH1->Add( h1 );
            h1->Delete();
	    
            TH1D* h2 = (*chanIt).GetTimeH1_B( 0 ); 
            beforeH1->Add( h2 );
            h2->Delete();
          }
	}
      }
  }
  
  c1->Clear();
  c1->Divide(2,2);
  after5mmH1->SetLineColor(2); 
  after10mmH1->SetLineColor(2);
  c1->cd(1);
  after5mmH1->SetLineColor(4);
  after5mmH1->Draw();
  before5mmH1->SetLineColor(2) ;
  before5mmH1->Draw("same"); 
  c1->cd(2);
  tzero5mmH1->Draw();
  c1->cd(3);
  after10mmH1->SetLineColor(4);
  after10mmH1->Draw();
  before10mmH1->SetLineColor(2) ;
  before10mmH1->Draw("same");
  c1->cd(4);
  tzero10mmH1->Draw();
}

void OTrTZeroCalib::Compare()
{
  ifstream ifsFi("caldata/tzero/TZeroFit981127.dat") ; 
  if ( ifsFi.is_open() <= 0 ) {
    cout << "file 1 not found" << endl;
    return;
  }
  ifstream ifsAv("caldata/tzero/TZeroAv.dat") ;
  if ( ifsAv.is_open() <= 0 ) {
    cout << "file 2 not found" << endl;
    return;
  }
  ifstream ifsTp("caldata/tzero/TZeroTP.dat") ; 
  if ( ifsTp.is_open() <= 0 ) {
    cout << "file 3 not found" << endl;
    return;
  }
  
  TH1F* tZeroTp05mmH1 = new TH1F("tZeroTp05mm","",50,-25,25) ;
  TH1F* tZeroFi05mmH1 = new TH1F("tZeroFi05mm","",50,-25,25) ;
  TH1F* tZeroAv05mmH1 = new TH1F("tZeroAv05mm","",50,-25,25) ;
  TH1F* tZeroTp10mmH1 = new TH1F("tZeroTp10mm","",50,-25,25) ;
  TH1F* tZeroFi10mmH1 = new TH1F("tZeroFi10mm","",50,-25,25) ;
  TH1F* tZeroAv10mmH1 = new TH1F("tZeroAv10mm","",50,-25,25) ;
  TH1F* deltaTZero05mmFT = new TH1F("deltaTZero05mmFT","",60,-25,25) ;
  TH1F* deltaTZero05mmFA = new TH1F("deltaTZero05mmFA","",60,-25,25) ;
  TH1F* deltaTZero05mmAT = new TH1F("deltaTZero05mmAT","",60,-25,25) ;
  TH1F* deltaTZero10mmFT = new TH1F("deltaTZero10mmFT","",60,-25,25) ;
  TH1F* deltaTZero10mmFA = new TH1F("deltaTZero10mmFA","",60,-25,25) ;
  TH1F* deltaTZero10mmAT = new TH1F("deltaTZero10mmAT","",60,-25,25) ;
  TH2F* TvsT05mmFT = new TH2F("TvsT05mmFT","",50,-25,25,50,-25,25) ;
  TH2F* TvsT05mmFA = new TH2F("TvsT05mmFA","",50,-25,25,50,-25,25) ;
  TH2F* TvsT05mmAT = new TH2F("TvsT05mmAT","",50,-25,25,50,-25,25) ;
  TH2F* TvsT10mmFT = new TH2F("TvsT10mmFT","",50,-25,25,50,-25,25) ;
  TH2F* TvsT10mmFA = new TH2F("TvsT10mmFA","",50,-25,25,50,-25,25) ;
  TH2F* TvsT10mmAT = new TH2F("TvsT10mmAT","",50,-25,25,50,-25,25) ;
 
  Int_t type,nTdc, tdcId ;
  float  t0Fi, t0Av, t0Tp ;
  float deltaFT, deltaFA, deltaAT ;
  OTrChannel* chanIt ;
  ifsFi >> type >> nTdc ;
  ifsAv >> type >> nTdc ;
  ifsTp >> type >> nTdc ;
  TCanvas* c1= new TCanvas("compareCanvas","compare") ;
  c1->Divide(3,3);
  c1->Modified();
  c1->Update();
  c1->Draw();
  int nplotted = 0;
  int ncanvas = 0;
  while(ifsFi >> tdcId && ifsAv >> tdcId && ifsTp >> tdcId ) {
    OTrTdcBoard* tdcIt = fDaq->GetTdcBoard(tdcId) ;
    for(Int_t asdId=0; asdId<OTrTdcBoard::GetNAsdBoards(); asdId++) {
      for(Int_t chanId=0; chanId<OTrAsdBoard::GetNChannels(); chanId++) {
        ifsFi >> t0Fi ;
        ifsAv >> t0Av ;
	ifsTp  >> t0Tp ;
	chanIt = &tdcIt->GetAsdBoards()[asdId].GetChannels()[chanId] ;

	if ( chanIt == 0 || (*chanIt).GetWire() == 0 ) continue;

	//	if ( (*chanIt).GetMask() > 100 ) {
	//cout << tdcId << " " << asdId << " " << chanId << " " 
	//     << (*chanIt).GetMask() << endl;
	//}

	// I added a t0Fi != 0 (Ivar)
	if(tdcIt && t0Tp>-90 && t0Av>-90 && t0Fi>-90 && t0Fi != 0 )
	  {
	    if ( !chanIt->GoodMask() ) continue; // use only good channels 
	    if((*chanIt).GetWire()) {
	      deltaFT = t0Fi - t0Tp ;
	      deltaFA = t0Fi - t0Av ;
	      deltaAT = t0Av - t0Tp ;
	      if( ((OTrGede*)(*chanIt).GetWire()->GetSection())->GetEPitch()
		  ==OTrGede::kFiveMM ) {
		if ( t0Fi < -12 ) {
		  cout << "t0Fi, t0Av, deltaFA = " 
		       << t0Fi << " "
		       << t0Av << " " 
		       << deltaFA << endl;
		  cout << tdcId << " " << asdId << " " << chanId << " "
		       << "wire = " << (*chanIt).GetWire()->GetId() << " " 
		       << "occ = " << (*chanIt).GetOccupancy() << " "
		       << "tzero = " << (*chanIt).GetTZero() << " "
		       << "av time = " << (*chanIt).GetAverageTime() << " " 
		       << "mask = " << (*chanIt).GetMask() << endl;
 		  
		  TH1* histo = (*chanIt).GetTimeH1() ;
		  nplotted++;
		  c1->cd(nplotted);
		  histo->Draw() ;
		  if (nplotted%9 == 0 ) {
		    c1->Update();
		    char psfile[256];
                    ncanvas++;
		    sprintf (psfile,"/hb/scratch/month/siccama/tdc_spectrum_%d%s",ncanvas,".ps");
		    //		    c1->Print(psfile);
		    getchar();
		    nplotted = 0;
		  }
		  //delete histo;
		}
		
		tZeroTp05mmH1->Fill(t0Tp) ;
		tZeroFi05mmH1->Fill(t0Fi) ;
		tZeroAv05mmH1->Fill(t0Av) ;
		deltaTZero05mmFT->Fill(deltaFT) ;
		deltaTZero05mmFA->Fill(deltaFA) ;
		deltaTZero05mmAT->Fill(deltaAT) ;
		TvsT05mmFT->Fill(t0Fi,t0Tp) ;
		TvsT05mmFA->Fill(t0Fi,t0Av) ;
		TvsT05mmAT->Fill(t0Av,t0Tp) ;
	      } else {
		tZeroTp10mmH1->Fill(t0Tp) ;
		tZeroFi10mmH1->Fill(t0Fi) ;
		tZeroAv10mmH1->Fill(t0Av) ;
		deltaTZero10mmFT->Fill(deltaFT) ;
		deltaTZero10mmFA->Fill(deltaFA) ;
		deltaTZero10mmAT->Fill(deltaAT) ;
		TvsT10mmFT->Fill(t0Fi,t0Tp) ;
		TvsT10mmFA->Fill(t0Fi,t0Av) ;
		TvsT10mmAT->Fill(t0Av,t0Tp) ;
	      };	     
	    };
	  };
      };
    };
  };

  c1->Divide(3,2) ;
  c1->cd(1) ;
  tZeroTp05mmH1->SetLineColor(2) ; tZeroTp05mmH1->Draw() ;
  tZeroFi05mmH1->SetLineColor(9) ; tZeroFi05mmH1->Draw("same") ;
  tZeroAv05mmH1->SetLineColor(5) ; tZeroAv05mmH1->Draw("same") ;
  c1->cd(4) ;
  tZeroTp10mmH1->SetLineColor(2) ; tZeroTp10mmH1->Draw() ;
  tZeroFi10mmH1->SetLineColor(9) ; tZeroFi10mmH1->Draw("same") ;
  tZeroAv10mmH1->SetLineColor(5) ; tZeroAv10mmH1->Draw("same") ;
  c1->cd(2) ;
  deltaTZero05mmFA->SetLineColor(8) ; deltaTZero05mmFA->Draw() ;
  deltaTZero05mmFT->SetLineColor(6) ; deltaTZero05mmFT->Draw("same") ;
  deltaTZero05mmAT->SetLineColor(44) ; deltaTZero05mmAT->Draw("same") ;
  c1->cd(5) ;
  deltaTZero10mmFA->SetLineColor(8) ; deltaTZero10mmFA->Draw() ;
  deltaTZero10mmFT->SetLineColor(6) ; deltaTZero10mmFT->Draw("same") ;
  deltaTZero10mmAT->SetLineColor(44) ; deltaTZero10mmAT->Draw("same") ;
  c1->cd(3) ;
  TvsT05mmFA->Draw() ;
  c1->cd(6) ;
  TvsT10mmFA->Draw() ;
  c1->Update() ;
}

void OTrTZeroCalib::CompareCalibrationFiles(char* file1, char* file2 )
{
  ifstream ifs1(file1) ; 
  if ( ifs1.is_open() <= 0 ) {
    cout << "file " << file1 << " not found" << endl;
    return;
  }
  ifstream ifs2(file2) ;
  if ( ifs2.is_open() <= 0 ) {
    cout << "file " << file2 << " not found" << endl;
    return;
  }

  float tmin = -25;
  float tmax =  25;
  float tmin1 = -25;
  float tmax1 =  25;
  float tmin2 = -25;
  float tmax2 =  25;
  TH1F* tone5     = new TH1F("tone5","",50,tmin,tmax) ;
  TH1F* ttwo5     = new TH1F("ttwo5","",50,tmin,tmax) ;
  TH1F* diff5     = new TH1F("diff5","",50,tmin1,tmax1) ;
  TH2F* diff5two  = new TH2F("diff5two","",50,tmin1,tmax1,50,tmin2,tmax2);

  TH1F* tone10    = new TH1F("tone10","",50,tmin,tmax) ;
  TH1F* ttwo10    = new TH1F("ttwo10","",50,tmin,tmax) ;
  TH1F* diff10    = new TH1F("diff10","",50,tmin1,tmax1) ;
  TH2F* diff10two = new TH2F("diff10two","",50,tmin1,tmax1,50,tmin2,tmax2);

  int type,nTdc;
  ifs1 >> type >> nTdc ;
  ifs2 >> type >> nTdc ;
 
  int tdcId ;
  OTrChannel* chanIt ;
  float t1,t2;

  TCanvas* c1= new TCanvas("compareCanvas","compare") ;
  c1->Divide(3,2);
  c1->Modified();
  c1->Update();
  c1->Draw();

  while ( ifs1 >> tdcId && ifs2 >> tdcId ) {
    cout << "tdcId = " << tdcId << endl;
    OTrTdcBoard* tdcIt = fDaq->GetTdcBoard(tdcId) ;
    for(Int_t asdId=0; asdId<OTrTdcBoard::GetNAsdBoards(); asdId++) {
      for(Int_t chanId=0; chanId<OTrAsdBoard::GetNChannels(); chanId++) {
        ifs1 >> t1 ;
        ifs2 >> t2 ;
	chanIt = &tdcIt->GetAsdBoards()[asdId].GetChannels()[chanId] ;
	
	if ( chanIt == 0 || (*chanIt).GetWire() == 0 ) continue;
	if ( !chanIt->GoodMask() ) continue; // use only good channels 
	if ( chanIt->GetWire()->GetSuperLayerId() != 
	     GetSuperLayerId() ) continue;

	if( tdcIt && t1>-90 && t2>-90 ) {
	  if ( ((OTrGede*)(*chanIt).GetWire()->GetSection())->GetEPitch()
	       == OTrGede::kFiveMM ) {
	    tone5->Fill(t1);
	    ttwo5->Fill(t2);
	    diff5->Fill(t1-t2) ;
	    diff5two->Fill(t1,t2);
	  } else {
	    tone10->Fill(t1);
	    ttwo10->Fill(t2);
 	    diff10->Fill(t1-t2) ;
	    diff10two->Fill(t1,t2);
	  }
	}
      }
    }
  }

  ttwo5->SetLineColor(2);
  ttwo10->SetLineColor(2);

  c1->cd(1) ;
  tone5->Draw();
  ttwo5->Draw("same");
  c1->cd(2);
  diff5->Draw();
  c1->cd(3);
  diff5two->Draw();

  c1->cd(4);
  tone10->Draw();
  ttwo10->Draw("same");
  c1->cd(5);
  diff10->Draw();
  c1->cd(6);
  diff10two->Draw();

  c1->Update() ;
};
#endif //USE_ROOT

/*
-- obsolete code --
void OTrTZeroCalib::CalibrationFromReference()
{
  const OTrChannel::TimeHistogramBin_t* timeHistogramReference5mm ;
  const OTrChannel::TimeHistogramBin_t* timeHistogramReference10mm ;
  
  bool converged=false ;
  HepVector Q(3) ; Q[0] = 0; Q[1] = 1; Q[2] = 0 ;
  HepVector dChi2dQ(3,0) ;
  HepSymMatrix d2Chi2dQ2(3,0) ;

  double& T0 = Q[0] ;
  double& alpha = Q[1] ;
  double& beta = Q[2] ;


  while(!converged) {
    for(bin=0; bin<OTrChannel::GetNTimeBins() ; bin++)
      {
	thisNref = Nref[(bin+Q[0])%OTrChannel::GetNTimeBins()] ;
	thisN    = N[bin] ;
	residual = (thisN*alpha + beta - thisNref) ;
	weight = 1 ; //(double)thisNref ;
	chi2 += weight*residual*residual ;
	dChi2dQ[0] += weight*residual* alpha * dNdT0[bin] ;
	dChi2dQ[1] += weight*residual* N[bin] ;
	dChi2dQ[2] += weight*residual ;
	d2Chi2Q2[0][0] += weight * alpha *
	  (alpha*dNdT0[bin] + residual * d2NdT0[bin]) ;
	d2Chi2Q2[0][1] += weight*dNdT0[bin] * 
	  (dNdT0[bin]*alpha + residual) ;
	d2Chi2Q2[0][2] += weight*
      }
 }

*/


