#ifndef MYPROFILE_HH
#define MYPROFILE_HH
#include <iostream.h>

/** MyProfile is a private implementation of a profile as described by the TProfile class in the ROOT package. **/
class MyProfile {
public:
  ///
  MyProfile() ;
  ///
  MyProfile(int, double, double) ;
  ///
  void Fill(double X, double Y, double Weight=1) ;
  ///
  class BinContent ;
  ///
  const BinContent& GetBinContent(int) const ;
  ///
  const int GetNoBins() const ;
  ///
  const double GetBinCenter(int) const ;
  ///
  void Reset() ;
  ///
  void Repair(int minno=5) ;
  ///
  void Print() ;
private:
  int fNoBins;
  double fXMin;
  double fXMax;
  BinContent* fXBins ;
  double tDeltaX;
};

class MyProfile::BinContent
{
private:
  int noEntries;
  double sumOfWeights;
  double sumOfYValues ;
  double sumOfYValues2 ;
  double sumOfXValues ;
  double sumOfXValues2 ;
public:
  BinContent():noEntries(0),sumOfWeights(0.),sumOfYValues(0.),sumOfYValues2(0.),
    sumOfXValues(0.),sumOfXValues2(0.) {} ;
  BinContent(int noe, double ws, double vs, double v2s):
    noEntries(noe),sumOfWeights(ws),sumOfYValues(vs),sumOfYValues2(v2s),
  sumOfXValues(0),sumOfXValues2(0.) {} ;
  void Add(double Yvalue, double weight=1, double Xvalue=1)
  {
    noEntries++ ;
    sumOfWeights+= weight ;
    sumOfYValues+= weight*Yvalue;
    sumOfYValues2+= weight*Yvalue*Yvalue;
    sumOfXValues+= weight*Xvalue;
    sumOfXValues2+= weight*Xvalue*Xvalue;
  };
  void Add(const BinContent& rhs)
  {
    noEntries += rhs.noEntries;
    sumOfWeights+= rhs.sumOfWeights ;
    sumOfYValues+= rhs.sumOfYValues;
    sumOfYValues2+= rhs.sumOfYValues2;
    sumOfXValues+= rhs.sumOfXValues;
    sumOfXValues2+= rhs.sumOfXValues2;
  }
  double GetFirstMoment() const {
    if(sumOfWeights>0.) return sumOfYValues/sumOfWeights;
    return 0. ;};
  double GetSecondMoment() const {
    if(sumOfWeights>0.) 
      return (sumOfYValues2 - sumOfYValues*sumOfYValues/sumOfWeights)/sumOfWeights ;
    return 0. ; };
  double GetBareSecondMoment() const {
    if(sumOfWeights>0.) 
      return sumOfYValues2/sumOfWeights ;
    return 0. ; };
  double GetXFirstMoment() const {
    if(sumOfWeights>0.) return sumOfXValues/sumOfWeights;
    return 0. ;};
  double GetXSecondMoment() const {
    if(sumOfWeights>0.) 
      return (sumOfXValues2 - sumOfXValues*sumOfXValues/sumOfWeights)/sumOfWeights ;
    return 0. ; };
  int GetNoEntries() const {return noEntries;} ;
  double GetSumOfWeights() const { return sumOfWeights; } ;
  double GetSumOfValues() const { return sumOfYValues; } ;
  double GetSumOfValues2() const { return sumOfYValues2; } ;
  void Reset() { noEntries=0;sumOfWeights=0;sumOfYValues=0;sumOfYValues2=0;
                 sumOfXValues=0;sumOfXValues2=0;} ;
  void Print() { cout << noEntries << " " << sumOfWeights << " " 
		      << sumOfYValues << " " <<  sumOfYValues2 << endl ; } ;
};

inline MyProfile::MyProfile() : fNoBins(1),fXMin(0),fXMax(1) {
  tDeltaX = (fXMax-fXMin)/fNoBins ;
  fXBins = new BinContent[fNoBins] ;
} ;
 
inline MyProfile::MyProfile(int noBins, double xMin, double xMax):
  fNoBins(noBins),fXMin(xMin),fXMax(xMax) {
  tDeltaX = (fXMax-fXMin)/fNoBins ;
  fXBins = new BinContent[fNoBins] ;
} ;

inline const MyProfile::BinContent& MyProfile::GetBinContent(int bin) const {
  return fXBins[bin] ; } ;

inline const int MyProfile::GetNoBins() const {
  return fNoBins ; } ;

inline const double MyProfile::GetBinCenter(int bin) const {
  return fXMin + tDeltaX * (bin+0.5) ; } ;

inline void MyProfile::Fill(double X, double Y, double Weight) {
  int xbin = int((X-fXMin)/tDeltaX) ;
  if(xbin<=0) xbin= 0;
  else if(xbin>=fNoBins) xbin = fNoBins-1;
  fXBins[xbin].Add(Y,Weight,X) ;
};

inline void MyProfile::Reset() {
  for(int i=0; i<GetNoBins(); i++) fXBins[i].Reset() ;
};

inline void MyProfile::Print() {
  for(int i=0; i<GetNoBins(); i++) fXBins[i].Print() ;
};


inline void MyProfile::Repair(int minno) {
  cout << "MyProfile::Repair() repaired bins: " ;
  int middlebin = GetNoBins()/2;
  int i;
  for(i=0; i<middlebin; i++) {
    bool repaired=false;
    int j=i+1;
    while(fXBins[i].GetNoEntries()<minno && j<GetNoBins()) {
      repaired = true;
      fXBins[i].Add(fXBins[j++]) ;
    };
    if(repaired) cout << i << " ";
  };

  for(i=middlebin; i<GetNoBins(); i++) {
    bool repaired=false;
    int j=i-1;
    while(fXBins[i].GetNoEntries()< minno&& j>=0) {
      repaired = true;
      fXBins[i].Add(fXBins[j--]) ;
    };
    if(repaired) cout << i << " ";
  };
  cout << endl;
};

#endif





